@@ -25,6 +25,7 @@ fn main() {
             direction,
             cwd,
             floating,
+            in_place,
             name,
             close_on_exit,
             start_suspended,
@@ -36,6 +37,7 @@ fn main() {
                 direction,
                 cwd,
                 floating,
+                in_place,
                 name,
                 close_on_exit,
                 start_suspended,
@@ -44,11 +46,34 @@ fn main() {
             commands::send_action_to_session(command_cli_action, opts.session, config);
             std::process::exit(0);
         }
+        if let Some(Command::Sessions(Sessions::Plugin {
+            url,
+            floating,
+            in_place,
+            configuration,
+        })) = opts.command
+        {
+            let command_cli_action = CliAction::NewPane {
+                command: vec![],
+                plugin: Some(url),
+                direction: None,
+                cwd: None,
+                floating,
+                in_place,
+                name: None,
+                close_on_exit: false,
+                start_suspended: false,
+                configuration,
+            };
+            commands::send_action_to_session(command_cli_action, opts.session, config);
+            std::process::exit(0);
+        }
         if let Some(Command::Sessions(Sessions::Edit {
             file,
             direction,
             line_number,
             floating,
+            in_place,
             cwd,
         })) = opts.command
         {
@@ -64,6 +89,7 @@ fn main() {
                 direction,
                 line_number,
                 floating,
+                in_place,
                 cwd,
             };
             commands::send_action_to_session(command_cli_action, opts.session, config);
@@ -83,14 +109,27 @@ fn main() {
         }
     }
 
-    if let Some(Command::Sessions(Sessions::ListSessions)) = opts.command {
-        commands::list_sessions();
+    if let Some(Command::Sessions(Sessions::ListSessions {
+        no_formatting,
+        short,
+    })) = opts.command
+    {
+        commands::list_sessions(no_formatting, short);
     } else if let Some(Command::Sessions(Sessions::KillAllSessions { yes })) = opts.command {
         commands::kill_all_sessions(yes);
     } else if let Some(Command::Sessions(Sessions::KillSession { ref target_session })) =
         opts.command
     {
         commands::kill_session(target_session);
+    } else if let Some(Command::Sessions(Sessions::DeleteAllSessions { yes, force })) = opts.command
+    {
+        commands::delete_all_sessions(yes, force);
+    } else if let Some(Command::Sessions(Sessions::DeleteSession {
+        ref target_session,
+        force,
+    })) = opts.command
+    {
+        commands::delete_session(target_session, force);
     } else if let Some(path) = opts.server {
         commands::start_server(path, opts.debug);
     } else {
