@@ -10,6 +10,7 @@ mod pty;
 mod pty_writer;
 mod route;
 mod screen;
+mod session_layout_metadata;
 mod terminal_bytes;
 mod thread_bus;
 mod ui;
@@ -43,6 +44,7 @@ use zellij_utils::{
     consts::{DEFAULT_SCROLL_BUFFER_SIZE, SCROLL_BUFFER_SIZE},
     data::{ConnectToSession, Event, PluginCapabilities},
     errors::{prelude::*, ContextType, ErrorInstruction, FatalError, ServerContext},
+    home::get_default_data_dir,
     input::{
         command::{RunCommand, TerminalAction},
         get_mode_info,
@@ -51,7 +53,6 @@ use zellij_utils::{
         plugins::PluginsConfig,
     },
     ipc::{ClientAttributes, ExitReason, ServerToClientMsg},
-    setup::get_default_data_dir,
 };
 
 pub type ClientId = u16;
@@ -780,7 +781,7 @@ fn init_session(
                 config_options.scrollback_editor.clone(),
             );
 
-            move || pty_thread_main(pty, layout).fatal()
+            move || pty_thread_main(pty, layout.clone()).fatal()
         })
         .unwrap();
 
@@ -801,6 +802,7 @@ fn init_session(
 
             let client_attributes_clone = client_attributes.clone();
             let debug = opts.debug;
+            let layout = layout.clone();
             move || {
                 screen_thread_main(
                     screen_bus,
@@ -808,6 +810,7 @@ fn init_session(
                     client_attributes_clone,
                     config_options,
                     debug,
+                    layout,
                 )
                 .fatal();
             }
@@ -910,12 +913,21 @@ fn init_session(
 
 #[cfg(not(feature = "singlepass"))]
 fn get_store() -> Store {
+    use wasmer::{BaseTunables, Cranelift, Engine, Pages, Target};
     log::info!("Compiling plugins using Cranelift");
-    Store::new(&wasmer::Universal::new(wasmer::Cranelift::default()).engine())
+
+    // workaround for https://github.com/bytecodealliance/wasmtime/security/advisories/GHSA-ff4p-7xrq-q5r8
+    let mut tunables = BaseTunables::for_target(&Target::default());
+    tunables.static_memory_bound = Pages(0);
+    let compiler = Cranelift::default();
+    let mut engine: Engine = compiler.into();
+    engine.set_tunables(tunables);
+
+    Store::new(engine)
 }
 
 #[cfg(feature = "singlepass")]
 fn get_store() -> Store {
     log::info!("Compiling plugins using Singlepass");
-    Store::new(&wasmer::Universal::new(wasmer::Singlepass::default()).engine())
+    Store::new(wasmer::Singlepass::default())
 }
