@@ -31,7 +31,7 @@ use zellij_utils::{
 };
 
 use std::{
-    collections::{BTreeMap, HashMap, HashSet},
+    collections::{BTreeMap, BTreeSet, HashMap},
     env,
     fs::File,
     io::Write,
@@ -183,6 +183,7 @@ fn handle_openpty(
             }
             command
                 .args(&cmd.args)
+                .env("ZELLIJ_PANE_ID", &format!("{}", terminal_id))
                 .pre_exec(move || -> std::io::Result<()> {
                     if libc::login_tty(pid_secondary) != 0 {
                         panic!("failed to set controlling terminal");
@@ -503,6 +504,14 @@ pub trait ServerOsApi: Send + Sync {
     fn load_palette(&self) -> Palette;
     /// Returns the current working directory for a given pid
     fn get_cwd(&self, pid: Pid) -> Option<PathBuf>;
+    /// Returns the current working directory for multiple pids
+    fn get_cwds(&self, _pids: Vec<Pid>) -> HashMap<Pid, PathBuf> {
+        HashMap::new()
+    }
+    /// Get a list of all running commands by their parent process id
+    fn get_all_cmds_by_ppid(&self) -> HashMap<String, Vec<String>> {
+        HashMap::new()
+    }
     /// Writes the given buffer to a string
     fn write_to_file(&mut self, buf: String, file: Option<String>) -> Result<()>;
 
@@ -557,6 +566,7 @@ impl ServerOsApi for ServerOsInputOutput {
         }
         Ok(())
     }
+    #[allow(unused_assignments)]
     fn spawn_terminal(
         &self,
         terminal_action: TerminalAction,
@@ -572,7 +582,7 @@ impl ServerOsApi for ServerOsInputOutput {
             .with_context(err_context)?;
         let mut terminal_id = None;
         {
-            let current_ids: HashSet<u32> = self
+            let current_ids: BTreeSet<u32> = self
                 .terminal_id_to_raw_fd
                 .lock()
                 .to_anyhow()
@@ -580,13 +590,7 @@ impl ServerOsApi for ServerOsInputOutput {
                 .keys()
                 .copied()
                 .collect();
-            for i in 0..u32::MAX {
-                let i = i as u32;
-                if !current_ids.contains(&i) {
-                    terminal_id = Some(i);
-                    break;
-                }
-            }
+            terminal_id = current_ids.last().map(|l| l + 1).or(Some(0));
         }
         match terminal_id {
             Some(terminal_id) => {
@@ -614,12 +618,13 @@ impl ServerOsApi for ServerOsInputOutput {
             None => Err(anyhow!("no more terminal IDs left to allocate")),
         }
     }
+    #[allow(unused_assignments)]
     fn reserve_terminal_id(&self) -> Result<u32> {
         let err_context = || "failed to reserve a terminal ID".to_string();
 
         let mut terminal_id = None;
         {
-            let current_ids: HashSet<u32> = self
+            let current_ids: BTreeSet<u32> = self
                 .terminal_id_to_raw_fd
                 .lock()
                 .to_anyhow()
@@ -627,13 +632,7 @@ impl ServerOsApi for ServerOsInputOutput {
                 .keys()
                 .copied()
                 .collect();
-            for i in 0..u32::MAX {
-                let i = i as u32;
-                if !current_ids.contains(&i) {
-                    terminal_id = Some(i);
-                    break;
-                }
-            }
+            terminal_id = current_ids.last().map(|l| l + 1).or(Some(0));
         }
         match terminal_id {
             Some(terminal_id) => {
@@ -755,6 +754,50 @@ impl ServerOsApi for ServerOsInputOutput {
         None
     }
 
+    fn get_cwds(&self, pids: Vec<Pid>) -> HashMap<Pid, PathBuf> {
+        let mut system_info = System::new();
+        // Update by minimizing information.
+        // See https://docs.rs/sysinfo/0.22.5/sysinfo/struct.ProcessRefreshKind.html#
+        system_info.refresh_processes_specifics(ProcessRefreshKind::default());
+
+        let mut cwds = HashMap::new();
+        for pid in pids {
+            if let Some(process) = system_info.process(pid.into()) {
+                let cwd = process.cwd();
+                let cwd_is_empty = cwd.iter().next().is_none();
+                if !cwd_is_empty {
+                    cwds.insert(pid, process.cwd().to_path_buf());
+                }
+            }
+        }
+        cwds
+    }
+    fn get_all_cmds_by_ppid(&self) -> HashMap<String, Vec<String>> {
+        // the key is the stringified ppid
+        let mut cmds = HashMap::new();
+        if let Some(output) = Command::new("ps")
+            .args(vec!["-ao", "ppid,args"])
+            .output()
+            .ok()
+        {
+            let output = String::from_utf8(output.stdout.clone())
+                .unwrap_or_else(|_| String::from_utf8_lossy(&output.stdout).to_string());
+            for line in output.lines() {
+                let line_parts: Vec<String> = line
+                    .trim()
+                    .split_ascii_whitespace()
+                    .map(|p| p.to_owned())
+                    .collect();
+                let mut line_parts = line_parts.into_iter();
+                let ppid = line_parts.next();
+                if let Some(ppid) = ppid {
+                    cmds.insert(ppid.into(), line_parts.collect());
+                }
+            }
+        }
+        cmds
+    }
+
     fn write_to_file(&mut self, buf: String, name: Option<String>) -> Result<()> {
         let err_context = || "failed to write to file".to_string();
 
