@@ -49,7 +49,9 @@ macro_rules! get_or_create_grid {
                 $self.link_handler.clone(),
                 $self.character_cell_size.clone(),
                 $self.sixel_image_store.clone(),
+                $self.style.clone(),
                 $self.debug,
+                $self.arrow_fonts,
             );
             grid.hide_cursor();
             grid
@@ -85,6 +87,7 @@ pub(crate) struct PluginPane {
     loading_indication: LoadingIndication,
     requesting_permissions: Option<PluginPermission>,
     debug: bool,
+    arrow_fonts: bool,
 }
 
 impl PluginPane {
@@ -103,6 +106,7 @@ impl PluginPane {
         style: Style,
         invoked_with: Option<Run>,
         debug: bool,
+        arrow_fonts: bool,
     ) -> Self {
         let loading_indication = LoadingIndication::new(title.clone()).with_colors(style.colors);
         let initial_loading_message = loading_indication.to_string();
@@ -134,6 +138,7 @@ impl PluginPane {
             loading_indication,
             requesting_permissions: None,
             debug,
+            arrow_fonts,
         };
         for client_id in currently_connected_clients {
             plugin.handle_plugin_bytes(client_id, initial_loading_message.as_bytes().to_vec());
@@ -616,6 +621,13 @@ impl Pane for PluginPane {
             self.pane_name.to_owned()
         }
     }
+    fn custom_title(&self) -> Option<String> {
+        if self.pane_name.is_empty() {
+            None
+        } else {
+            Some(self.pane_name.clone())
+        }
+    }
     fn rename(&mut self, buf: Vec<u8>) {
         self.pane_name = String::from_utf8_lossy(&buf).to_string();
         self.set_should_render(true);
