@@ -21,6 +21,7 @@ pub const EMPTY_TERMINAL_CHARACTER: TerminalCharacter = TerminalCharacter {
 pub const RESET_STYLES: CharacterStyles = CharacterStyles {
     foreground: Some(AnsiCode::Reset),
     background: Some(AnsiCode::Reset),
+    underline_color: Some(AnsiCode::Reset),
     strike: Some(AnsiCode::Reset),
     hidden: Some(AnsiCode::Reset),
     reverse: Some(AnsiCode::Reset),
@@ -31,6 +32,7 @@ pub const RESET_STYLES: CharacterStyles = CharacterStyles {
     dim: Some(AnsiCode::Reset),
     italic: Some(AnsiCode::Reset),
     link_anchor: Some(LinkAnchor::End),
+    styled_underlines_enabled: false,
 };
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -40,6 +42,15 @@ pub enum AnsiCode {
     NamedColor(NamedColor),
     RgbCode((u8, u8, u8)),
     ColorIndex(u8),
+    Underline(Option<AnsiStyledUnderline>),
+}
+
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+pub enum AnsiStyledUnderline {
+    Double,
+    Undercurl,
+    Underdotted,
+    Underdashed,
 }
 
 impl From<PaletteColor> for AnsiCode {
@@ -122,6 +133,7 @@ impl NamedColor {
 pub struct CharacterStyles {
     pub foreground: Option<AnsiCode>,
     pub background: Option<AnsiCode>,
+    pub underline_color: Option<AnsiCode>,
     pub strike: Option<AnsiCode>,
     pub hidden: Option<AnsiCode>,
     pub reverse: Option<AnsiCode>,
@@ -132,6 +144,7 @@ pub struct CharacterStyles {
     pub dim: Option<AnsiCode>,
     pub italic: Option<AnsiCode>,
     pub link_anchor: Option<LinkAnchor>,
+    pub styled_underlines_enabled: bool,
 }
 
 impl CharacterStyles {
@@ -146,6 +159,10 @@ impl CharacterStyles {
         self.background = background_code;
         self
     }
+    pub fn underline_color(mut self, underline_color_code: Option<AnsiCode>) -> Self {
+        self.underline_color = underline_color_code;
+        self
+    }
     pub fn bold(mut self, bold_code: Option<AnsiCode>) -> Self {
         self.bold = bold_code;
         self
@@ -186,9 +203,14 @@ impl CharacterStyles {
         self.link_anchor = link_anchor;
         self
     }
+    pub fn enable_styled_underlines(mut self, enabled: bool) -> Self {
+        self.styled_underlines_enabled = enabled;
+        self
+    }
     pub fn clear(&mut self) {
         self.foreground = None;
         self.background = None;
+        self.underline_color = None;
         self.strike = None;
         self.hidden = None;
         self.reverse = None;
@@ -215,7 +237,8 @@ impl CharacterStyles {
         }
 
         // create diff from all changed styles
-        let mut diff = CharacterStyles::new();
+        let mut diff =
+            CharacterStyles::new().enable_styled_underlines(self.styled_underlines_enabled);
 
         if self.foreground != new_styles.foreground {
             diff.foreground = new_styles.foreground;
@@ -223,6 +246,9 @@ impl CharacterStyles {
         if self.background != new_styles.background {
             diff.background = new_styles.background;
         }
+        if self.underline_color != new_styles.underline_color {
+            diff.underline_color = new_styles.underline_color;
+        }
         if self.strike != new_styles.strike {
             diff.strike = new_styles.strike;
         }
@@ -274,6 +300,7 @@ impl CharacterStyles {
     pub fn reset_all(&mut self) {
         self.foreground = Some(AnsiCode::Reset);
         self.background = Some(AnsiCode::Reset);
+        self.underline_color = Some(AnsiCode::Reset);
         self.bold = Some(AnsiCode::Reset);
         self.dim = Some(AnsiCode::Reset);
         self.italic = Some(AnsiCode::Reset);
@@ -291,7 +318,28 @@ impl CharacterStyles {
                 [1] => *self = self.bold(Some(AnsiCode::On)),
                 [2] => *self = self.dim(Some(AnsiCode::On)),
                 [3] => *self = self.italic(Some(AnsiCode::On)),
-                [4] => *self = self.underline(Some(AnsiCode::On)),
+                [4, 0] => *self = self.underline(Some(AnsiCode::Reset)),
+                [4, 1] => *self = self.underline(Some(AnsiCode::Underline(None))),
+                [4, 2] => {
+                    *self =
+                        self.underline(Some(AnsiCode::Underline(Some(AnsiStyledUnderline::Double))))
+                },
+                [4, 3] => {
+                    *self = self.underline(Some(AnsiCode::Underline(Some(
+                        AnsiStyledUnderline::Undercurl,
+                    ))))
+                },
+                [4, 4] => {
+                    *self = self.underline(Some(AnsiCode::Underline(Some(
+                        AnsiStyledUnderline::Underdotted,
+                    ))))
+                },
+                [4, 5] => {
+                    *self = self.underline(Some(AnsiCode::Underline(Some(
+                        AnsiStyledUnderline::Underdashed,
+                    ))))
+                },
+                [4] => *self = self.underline(Some(AnsiCode::Underline(None))),
                 [5] => *self = self.blink_slow(Some(AnsiCode::On)),
                 [6] => *self = self.blink_fast(Some(AnsiCode::On)),
                 [7] => *self = self.reverse(Some(AnsiCode::On)),
@@ -357,6 +405,21 @@ impl CharacterStyles {
                     }
                 },
                 [49] => *self = self.background(Some(AnsiCode::Reset)),
+                [58] => {
+                    let mut iter = params.map(|param| param[0]);
+                    if let Some(ansi_code) = parse_sgr_color(&mut iter) {
+                        *self = self.underline_color(Some(ansi_code));
+                    }
+                },
+                [58, params @ ..] => {
+                    let rgb_start = if params.len() > 4 { 2 } else { 1 };
+                    let rgb_iter = params[rgb_start..].iter().copied();
+                    let mut iter = std::iter::once(params[0]).chain(rgb_iter);
+                    if let Some(ansi_code) = parse_sgr_color(&mut iter) {
+                        *self = self.underline_color(Some(ansi_code));
+                    }
+                },
+                [59] => *self = self.underline_color(Some(AnsiCode::Reset)),
                 [90] => {
                     *self = self.foreground(Some(AnsiCode::NamedColor(NamedColor::BrightBlack)))
                 },
@@ -409,6 +472,7 @@ impl Display for CharacterStyles {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         if self.foreground == Some(AnsiCode::Reset)
             && self.background == Some(AnsiCode::Reset)
+            && self.underline_color == Some(AnsiCode::Reset)
             && self.strike == Some(AnsiCode::Reset)
             && self.hidden == Some(AnsiCode::Reset)
             && self.reverse == Some(AnsiCode::Reset)
@@ -456,6 +520,22 @@ impl Display for CharacterStyles {
                 _ => {},
             }
         }
+        if self.styled_underlines_enabled {
+            if let Some(ansi_code) = self.underline_color {
+                match ansi_code {
+                    AnsiCode::RgbCode((r, g, b)) => {
+                        write!(f, "\u{1b}[58;2;{};{};{}m", r, g, b)?;
+                    },
+                    AnsiCode::ColorIndex(color_index) => {
+                        write!(f, "\u{1b}[58;5;{}m", color_index)?;
+                    },
+                    AnsiCode::Reset => {
+                        write!(f, "\u{1b}[59m")?;
+                    },
+                    _ => {},
+                }
+            };
+        }
         if let Some(ansi_code) = self.strike {
             match ansi_code {
                 AnsiCode::On => {
@@ -529,15 +609,34 @@ impl Display for CharacterStyles {
         // otherwise
         if let Some(ansi_code) = self.underline {
             match ansi_code {
-                AnsiCode::On => {
+                AnsiCode::Underline(None) => {
                     write!(f, "\u{1b}[4m")?;
                 },
+                AnsiCode::Underline(Some(styled)) => {
+                    if self.styled_underlines_enabled {
+                        match styled {
+                            AnsiStyledUnderline::Double => {
+                                write!(f, "\u{1b}[4:2m")?;
+                            },
+                            AnsiStyledUnderline::Undercurl => {
+                                write!(f, "\u{1b}[4:3m")?;
+                            },
+                            AnsiStyledUnderline::Underdotted => {
+                                write!(f, "\u{1b}[4:4m")?;
+                            },
+                            AnsiStyledUnderline::Underdashed => {
+                                write!(f, "\u{1b}[4:5m")?;
+                            },
+                        }
+                    }
+                },
                 AnsiCode::Reset => {
                     write!(f, "\u{1b}[24m")?;
                 },
                 _ => {},
             }
         }
+
         if let Some(ansi_code) = self.dim {
             match ansi_code {
                 AnsiCode::On => {
@@ -773,21 +872,25 @@ pub fn render_first_run_banner(
 
             let controls_bare_text_first_part = "<";
             let enter_bare_text = "ENTER";
-            let controls_bare_text_second_part = "> to run, <";
+            let controls_bare_text_second_part = "> run, <";
+            let esc_bare_text = "ESC";
+            let controls_bare_text_third_part = "> drop to shell, <";
             let ctrl_c_bare_text = "Ctrl-c";
-            let controls_bare_text_third_part = "> to exit";
+            let controls_bare_text_fourth_part = "> exit";
             let controls_color = RESET_STYLES
                 .foreground(Some(AnsiCode::from(style.colors.orange)))
                 .bold(Some(AnsiCode::On));
             let controls_line_length = controls_bare_text_first_part.len()
                 + enter_bare_text.len()
                 + controls_bare_text_second_part.len()
+                + esc_bare_text.len()
+                + controls_bare_text_third_part.len()
                 + ctrl_c_bare_text.len()
-                + controls_bare_text_third_part.len();
+                + controls_bare_text_fourth_part.len();
             let controls_column_start_position =
                 middle_column.saturating_sub(controls_line_length / 2);
             let controls_line = format!(
-                "\u{1b}[{};{}H{}<{}{}{}{}> to run, <{}{}{}{}> to exit",
+                "\u{1b}[{};{}H{}<{}{}{}{}> run, <{}{}{}{}> drop to shell, <{}{}{}{}> exit",
                 middle_row + 2,
                 controls_column_start_position,
                 bold_text,
@@ -796,6 +899,10 @@ pub fn render_first_run_banner(
                 RESET_STYLES,
                 bold_text,
                 controls_color,
+                esc_bare_text,
+                RESET_STYLES,
+                bold_text,
+                controls_color,
                 ctrl_c_bare_text,
                 RESET_STYLES,
                 bold_text
@@ -817,21 +924,25 @@ pub fn render_first_run_banner(
 
             let controls_bare_text_first_part = "<";
             let enter_bare_text = "ENTER";
-            let controls_bare_text_second_part = "> to run, <";
+            let controls_bare_text_second_part = "> run, <";
+            let esc_bare_text = "ESC";
+            let controls_bare_text_third_part = "> drop to shell, <";
             let ctrl_c_bare_text = "Ctrl-c";
-            let controls_bare_text_third_part = "> to exit";
+            let controls_bare_text_fourth_part = "> exit";
             let controls_color = RESET_STYLES
                 .foreground(Some(AnsiCode::from(style.colors.orange)))
                 .bold(Some(AnsiCode::On));
             let controls_line_length = controls_bare_text_first_part.len()
                 + enter_bare_text.len()
                 + controls_bare_text_second_part.len()
+                + esc_bare_text.len()
+                + controls_bare_text_third_part.len()
                 + ctrl_c_bare_text.len()
-                + controls_bare_text_third_part.len();
+                + controls_bare_text_fourth_part.len();
             let controls_column_start_position =
                 middle_column.saturating_sub(controls_line_length / 2);
             let controls_line = format!(
-                "\u{1b}[{};{}H{}<{}{}{}{}> to run, <{}{}{}{}> to exit",
+                "\u{1b}[{};{}H{}<{}{}{}{}> run, <{}{}{}{}> drop to shell, <{}{}{}{}> exit",
                 middle_row + 2,
                 controls_column_start_position,
                 bold_text,
@@ -840,6 +951,10 @@ pub fn render_first_run_banner(
                 RESET_STYLES,
                 bold_text,
                 controls_color,
+                esc_bare_text,
+                RESET_STYLES,
+                bold_text,
+                controls_color,
                 ctrl_c_bare_text,
                 RESET_STYLES,
                 bold_text
