@@ -40,6 +40,7 @@ const END_KEY: &[u8] = &[27, 91, 70];
 const BRACKETED_PASTE_BEGIN: &[u8] = &[27, 91, 50, 48, 48, 126];
 const BRACKETED_PASTE_END: &[u8] = &[27, 91, 50, 48, 49, 126];
 const ENTER_NEWLINE: &[u8] = &[10];
+const ESC: &[u8] = &[27];
 const ENTER_CARRIAGE_RETURN: &[u8] = &[13];
 const SPACE: &[u8] = &[32];
 const CTRL_C: &[u8] = &[3]; // TODO: check this to be sure it fits all types of CTRL_C (with mac, etc)
@@ -114,6 +115,8 @@ pub struct TerminalPane {
     // held on startup and can possibly be used to display some errors
     pane_frame_color_override: Option<(PaletteColor, Option<String>)>,
     invoked_with: Option<Run>,
+    #[allow(dead_code)]
+    arrow_fonts: bool,
 }
 
 impl Pane for TerminalPane {
@@ -192,6 +195,13 @@ impl Pane for TerminalPane {
                     self.remove_banner();
                     Some(AdjustedInput::ReRunCommandInThisPane(run_command))
                 },
+                ESC => {
+                    self.is_held = None;
+                    self.grid.reset_terminal_state();
+                    self.set_should_render(true);
+                    self.remove_banner();
+                    Some(AdjustedInput::DropToShellInThisPane)
+                },
                 CTRL_C => Some(AdjustedInput::CloseThisPane),
                 _ => None,
             }
@@ -732,6 +742,13 @@ impl Pane for TerminalPane {
             self.pane_name.to_owned()
         }
     }
+    fn custom_title(&self) -> Option<String> {
+        if self.pane_name.is_empty() {
+            None
+        } else {
+            Some(self.pane_name.clone())
+        }
+    }
     fn exit_status(&self) -> Option<i32> {
         self.is_held
             .as_ref()
@@ -750,6 +767,9 @@ impl Pane for TerminalPane {
         self.pane_name = String::from_utf8_lossy(&buf).to_string();
         self.set_should_render(true);
     }
+    fn serialize(&self, scrollback_lines_to_serialize: Option<usize>) -> Option<String> {
+        self.grid.serialize(scrollback_lines_to_serialize)
+    }
 }
 
 impl TerminalPane {
@@ -768,6 +788,7 @@ impl TerminalPane {
         initial_pane_title: Option<String>,
         invoked_with: Option<Run>,
         debug: bool,
+        arrow_fonts: bool,
     ) -> TerminalPane {
         let initial_pane_title =
             initial_pane_title.unwrap_or_else(|| format!("Pane #{}", pane_index));
@@ -779,7 +800,9 @@ impl TerminalPane {
             link_handler,
             character_cell_size,
             sixel_image_store,
+            style.clone(),
             debug,
+            arrow_fonts,
         );
         TerminalPane {
             frame: HashMap::new(),
@@ -804,6 +827,7 @@ impl TerminalPane {
             banner: None,
             pane_frame_color_override: None,
             invoked_with,
+            arrow_fonts,
         }
     }
     pub fn get_x(&self) -> usize {
