@@ -9,15 +9,20 @@ use std::{
     collections::{HashMap, HashSet},
     fs,
     path::PathBuf,
+    sync::{Arc, Mutex},
+    time::Duration,
 };
 use wasmer::Store;
 
+use crate::panes::PaneId;
 use crate::screen::ScreenInstruction;
+use crate::session_layout_metadata::SessionLayoutMetadata;
 use crate::{pty::PtyInstruction, thread_bus::Bus, ClientId, ServerInstruction};
 
 use wasm_bridge::WasmBridge;
 
 use zellij_utils::{
+    async_std::{channel, future::timeout, task},
     data::{Event, EventType, PermissionStatus, PermissionType, PluginCapabilities},
     errors::{prelude::*, ContextType, PluginContext},
     input::{
@@ -38,11 +43,14 @@ pub type PluginId = u32;
 pub enum PluginInstruction {
     Load(
         Option<bool>,   // should float
+        bool,           // should be opened in place
         Option<String>, // pane title
         RunPlugin,
-        usize, // tab index
+        usize,          // tab index
+        Option<PaneId>, // pane id to replace if this is to be opened "in-place"
         ClientId,
         Size,
+        Option<PathBuf>, // cwd
     ),
     Update(Vec<(Option<PluginId>, Option<ClientId>, Event)>), // Focused plugin / broadcast, client_id, event data
     Unload(PluginId),                                         // plugin_id
@@ -89,6 +97,8 @@ pub enum PluginInstruction {
         PermissionStatus,
         Option<PathBuf>,
     ),
+    DumpLayout(SessionLayoutMetadata, ClientId),
+    LogLayoutToHd(SessionLayoutMetadata),
     Exit,
 }
 
@@ -118,6 +128,8 @@ impl From<&PluginInstruction> for PluginContext {
             PluginInstruction::PermissionRequestResult(..) => {
                 PluginContext::PermissionRequestResult
             },
+            PluginInstruction::DumpLayout(..) => PluginContext::DumpLayout,
+            PluginInstruction::LogLayoutToHd(..) => PluginContext::LogLayoutToHd,
         }
     }
 }
@@ -139,6 +151,12 @@ pub(crate) fn plugin_thread_main(
     let plugin_dir = data_dir.join("plugins/");
     let plugin_global_data_dir = plugin_dir.join("data");
 
+    let store = Arc::new(Mutex::new(store));
+
+    // use this channel to ensure that tasks spawned from this thread terminate before exiting
+    // https://tokio.rs/tokio/topics/shutdown#waiting-for-things-to-finish-shutting-down
+    let (shutdown_send, shutdown_receive) = channel::bounded::<()>(1);
+
     let mut wasm_bridge = WasmBridge::new(
         plugins,
         bus.senders.clone(),
@@ -156,24 +174,35 @@ pub(crate) fn plugin_thread_main(
         let (event, mut err_ctx) = bus.recv().expect("failed to receive event on channel");
         err_ctx.add_call(ContextType::Plugin((&event).into()));
         match event {
-            PluginInstruction::Load(should_float, pane_title, run, tab_index, client_id, size) => {
-                match wasm_bridge.load_plugin(&run, tab_index, size, Some(client_id)) {
-                    Ok(plugin_id) => {
-                        drop(bus.senders.send_to_screen(ScreenInstruction::AddPlugin(
-                            should_float,
-                            run,
-                            pane_title,
-                            tab_index,
-                            plugin_id,
-                        )));
-                    },
-                    Err(e) => {
-                        log::error!("Failed to load plugin: {e}");
-                    },
-                }
+            PluginInstruction::Load(
+                should_float,
+                should_be_open_in_place,
+                pane_title,
+                run,
+                tab_index,
+                pane_id_to_replace,
+                client_id,
+                size,
+                cwd,
+            ) => match wasm_bridge.load_plugin(&run, tab_index, size, cwd, Some(client_id)) {
+                Ok(plugin_id) => {
+                    drop(bus.senders.send_to_screen(ScreenInstruction::AddPlugin(
+                        should_float,
+                        should_be_open_in_place,
+                        run,
+                        pane_title,
+                        tab_index,
+                        plugin_id,
+                        pane_id_to_replace,
+                        Some(client_id),
+                    )));
+                },
+                Err(e) => {
+                    log::error!("Failed to load plugin: {e}");
+                },
             },
             PluginInstruction::Update(updates) => {
-                wasm_bridge.update_plugins(updates)?;
+                wasm_bridge.update_plugins(updates, shutdown_send.clone())?;
             },
             PluginInstruction::Unload(pid) => {
                 wasm_bridge.unload_plugin(pid)?;
@@ -190,14 +219,18 @@ pub(crate) fn plugin_thread_main(
                             log::warn!("Plugin {} not found, starting it instead", run.location);
                             // we intentionally do not provide the client_id here because it belongs to
                             // the cli who spawned the command and is not an existing client_id
-                            match wasm_bridge.load_plugin(&run, tab_index, size, None) {
+                            match wasm_bridge.load_plugin(&run, tab_index, size, None, None) {
                                 Ok(plugin_id) => {
+                                    let should_be_open_in_place = false;
                                     drop(bus.senders.send_to_screen(ScreenInstruction::AddPlugin(
                                         should_float,
+                                        should_be_open_in_place,
                                         run,
                                         pane_title,
                                         tab_index,
                                         plugin_id,
+                                        None,
+                                        None,
                                     )));
                                 },
                                 Err(e) => {
@@ -212,7 +245,7 @@ pub(crate) fn plugin_thread_main(
                 }
             },
             PluginInstruction::Resize(pid, new_columns, new_rows) => {
-                wasm_bridge.resize_plugin(pid, new_columns, new_rows)?;
+                wasm_bridge.resize_plugin(pid, new_columns, new_rows, shutdown_send.clone())?;
             },
             PluginInstruction::AddClient(client_id) => {
                 wasm_bridge.add_client(client_id)?;
@@ -250,8 +283,13 @@ pub(crate) fn plugin_thread_main(
                 extracted_run_instructions.append(&mut extracted_floating_plugins);
                 for run_instruction in extracted_run_instructions {
                     if let Some(Run::Plugin(run)) = run_instruction {
-                        let plugin_id =
-                            wasm_bridge.load_plugin(&run, tab_index, size, Some(client_id))?;
+                        let plugin_id = wasm_bridge.load_plugin(
+                            &run,
+                            tab_index,
+                            size,
+                            None,
+                            Some(client_id),
+                        )?;
                         plugin_ids
                             .entry((run.location, run.configuration))
                             .or_default()
@@ -269,7 +307,7 @@ pub(crate) fn plugin_thread_main(
                 )));
             },
             PluginInstruction::ApplyCachedEvents(plugin_id) => {
-                wasm_bridge.apply_cached_events(plugin_id)?;
+                wasm_bridge.apply_cached_events(plugin_id, shutdown_send.clone())?;
             },
             PluginInstruction::ApplyCachedWorkerMessages(plugin_id) => {
                 wasm_bridge.apply_cached_worker_messages(plugin_id)?;
@@ -293,7 +331,7 @@ pub(crate) fn plugin_thread_main(
                     Some(client_id),
                     Event::CustomMessage(message, payload),
                 )];
-                wasm_bridge.update_plugins(updates)?;
+                wasm_bridge.update_plugins(updates, shutdown_send.clone())?;
             },
             PluginInstruction::PluginSubscribedToEvents(_plugin_id, _client_id, events) => {
                 for event in events {
@@ -328,16 +366,42 @@ pub(crate) fn plugin_thread_main(
                     client_id,
                     Event::PermissionRequestResult(status),
                 )];
-                wasm_bridge.update_plugins(updates)?;
+                wasm_bridge.update_plugins(updates, shutdown_send.clone())?;
+            },
+            PluginInstruction::DumpLayout(mut session_layout_metadata, client_id) => {
+                populate_session_layout_metadata(&mut session_layout_metadata, &wasm_bridge);
+                drop(bus.senders.send_to_pty(PtyInstruction::DumpLayout(
+                    session_layout_metadata,
+                    client_id,
+                )));
+            },
+            PluginInstruction::LogLayoutToHd(mut session_layout_metadata) => {
+                populate_session_layout_metadata(&mut session_layout_metadata, &wasm_bridge);
+                drop(
+                    bus.senders
+                        .send_to_pty(PtyInstruction::LogLayoutToHd(session_layout_metadata)),
+                );
             },
             PluginInstruction::Exit => {
-                wasm_bridge.cleanup();
                 break;
             },
         }
     }
     info!("wasm main thread exits");
 
+    // first drop our sender, then call recv.
+    // once all senders are dropped or the timeout is reached, recv will return an error, that we ignore
+
+    drop(shutdown_send);
+    task::block_on(async {
+        let result = timeout(EXIT_TIMEOUT, shutdown_receive.recv()).await;
+        if let Err(err) = result {
+            log::error!("timeout waiting for plugin tasks to finish: {}", err);
+        }
+    });
+
+    wasm_bridge.cleanup();
+
     fs::remove_dir_all(&plugin_global_data_dir)
         .or_else(|err| {
             if err.kind() == std::io::ErrorKind::NotFound {
@@ -350,6 +414,26 @@ pub(crate) fn plugin_thread_main(
         .context("failed to cleanup plugin data directory")
 }
 
+fn populate_session_layout_metadata(
+    session_layout_metadata: &mut SessionLayoutMetadata,
+    wasm_bridge: &WasmBridge,
+) {
+    let plugin_ids = session_layout_metadata.all_plugin_ids();
+    let mut plugin_ids_to_cmds: HashMap<u32, RunPlugin> = HashMap::new();
+    for plugin_id in plugin_ids {
+        let plugin_cmd = wasm_bridge.run_plugin_of_plugin_id(plugin_id);
+        match plugin_cmd {
+            Some(plugin_cmd) => {
+                plugin_ids_to_cmds.insert(plugin_id, plugin_cmd.clone());
+            },
+            None => log::error!("Plugin with id: {plugin_id} not found"),
+        }
+    }
+    session_layout_metadata.update_plugin_cmds(plugin_ids_to_cmds);
+}
+
+const EXIT_TIMEOUT: Duration = Duration::from_secs(3);
+
 #[path = "./unit/plugin_tests.rs"]
 #[cfg(test)]
 mod plugin_tests;
