@@ -5,7 +5,7 @@ use std::{
     path::PathBuf,
     sync::{Arc, Mutex},
 };
-use wasmer::Instance;
+use wasmer::{Instance, Store};
 use wasmer_wasi::WasiEnv;
 
 use crate::{thread_bus::ThreadSenders, ClientId};
@@ -15,7 +15,7 @@ use zellij_utils::{
     data::EventType,
     data::PluginCapabilities,
     input::command::TerminalAction,
-    input::layout::{Layout, RunPluginLocation},
+    input::layout::{Layout, RunPlugin, RunPluginLocation},
     input::plugins::PluginConfig,
     ipc::ClientAttributes,
 };
@@ -185,6 +185,28 @@ impl PluginMap {
             (running_plugin, subscriptions, running_workers),
         );
     }
+    pub fn run_plugin_of_plugin_id(&self, plugin_id: PluginId) -> Option<RunPlugin> {
+        self.plugin_assets
+            .iter()
+            .find_map(|((p_id, _), (running_plugin, _, _))| {
+                if *p_id == plugin_id {
+                    let running_plugin = running_plugin.lock().unwrap();
+                    let run_plugin_location = running_plugin.plugin_env.plugin.location.clone();
+                    let run_plugin_configuration = running_plugin
+                        .plugin_env
+                        .plugin
+                        .userspace_configuration
+                        .clone();
+                    Some(RunPlugin {
+                        _allow_exec_host_cmd: false,
+                        location: run_plugin_location,
+                        configuration: run_plugin_configuration,
+                    })
+                } else {
+                    None
+                }
+            })
+    }
 }
 
 pub type Subscriptions = HashSet<EventType>;
@@ -228,6 +250,7 @@ pub enum AtomicEvent {
 }
 
 pub struct RunningPlugin {
+    pub store: Store,
     pub instance: Instance,
     pub plugin_env: PluginEnv,
     pub rows: usize,
@@ -237,8 +260,15 @@ pub struct RunningPlugin {
 }
 
 impl RunningPlugin {
-    pub fn new(instance: Instance, plugin_env: PluginEnv, rows: usize, columns: usize) -> Self {
+    pub fn new(
+        store: Store,
+        instance: Instance,
+        plugin_env: PluginEnv,
+        rows: usize,
+        columns: usize,
+    ) -> Self {
         RunningPlugin {
+            store,
             instance,
             plugin_env,
             rows,
