@@ -4,6 +4,7 @@ use crate::plugins::plugin_map::{AtomicEvent, PluginEnv, PluginMap, RunningPlugi
 use crate::plugins::plugin_worker::MessageToWorker;
 use crate::plugins::watch_filesystem::watch_filesystem;
 use crate::plugins::zellij_exports::{wasi_read_string, wasi_write_object};
+use highway::{HighwayHash, PortableHash};
 use log::info;
 use std::{
     collections::{HashMap, HashSet},
@@ -11,9 +12,13 @@ use std::{
     str::FromStr,
     sync::{Arc, Mutex},
 };
-use wasmer::{Instance, Module, Store, Value};
+use wasmer::{Module, Store, Value};
+use zellij_utils::async_channel::Sender;
 use zellij_utils::async_std::task::{self, JoinHandle};
+use zellij_utils::consts::ZELLIJ_CACHE_DIR;
 use zellij_utils::data::{PermissionStatus, PermissionType};
+use zellij_utils::downloader::download::Download;
+use zellij_utils::downloader::Downloader;
 use zellij_utils::input::permission::PermissionCache;
 use zellij_utils::notify_debouncer_full::{notify::RecommendedWatcher, Debouncer, FileIdMap};
 use zellij_utils::plugin_api::event::ProtobufEvent;
@@ -40,7 +45,7 @@ pub struct WasmBridge {
     connected_clients: Arc<Mutex<Vec<ClientId>>>,
     plugins: PluginsConfig,
     senders: ThreadSenders,
-    store: Store,
+    store: Arc<Mutex<Store>>,
     plugin_dir: PathBuf,
     plugin_cache: Arc<Mutex<HashMap<PathBuf, Module>>>,
     plugin_map: Arc<Mutex<PluginMap>>,
@@ -66,7 +71,7 @@ impl WasmBridge {
     pub fn new(
         plugins: PluginsConfig,
         senders: ThreadSenders,
-        store: Store,
+        store: Arc<Mutex<Store>>,
         plugin_dir: PathBuf,
         path_to_default_shell: PathBuf,
         zellij_cwd: PathBuf,
@@ -108,6 +113,7 @@ impl WasmBridge {
         run: &RunPlugin,
         tab_index: usize,
         size: Size,
+        cwd: Option<PathBuf>,
         client_id: Option<ClientId>,
     ) -> Result<PluginId> {
         // returns the plugin id
@@ -128,7 +134,7 @@ impl WasmBridge {
 
         let plugin_id = self.next_plugin_id;
 
-        let plugin = self
+        let mut plugin = self
             .plugins
             .get(run)
             .with_context(|| format!("failed to resolve plugin {run:?}"))
@@ -148,7 +154,7 @@ impl WasmBridge {
             let plugin_map = self.plugin_map.clone();
             let connected_clients = self.connected_clients.clone();
             let path_to_default_shell = self.path_to_default_shell.clone();
-            let zellij_cwd = self.zellij_cwd.clone();
+            let zellij_cwd = cwd.unwrap_or_else(|| self.zellij_cwd.clone());
             let capabilities = self.capabilities.clone();
             let client_attributes = self.client_attributes.clone();
             let default_shell = self.default_shell.clone();
@@ -157,6 +163,33 @@ impl WasmBridge {
                 let _ =
                     senders.send_to_background_jobs(BackgroundJob::AnimatePluginLoading(plugin_id));
                 let mut loading_indication = LoadingIndication::new(plugin_name.clone());
+
+                if let RunPluginLocation::Remote(url) = &plugin.location {
+                    let download = Download::from(url);
+
+                    let hash: String = PortableHash::default()
+                        .hash128(download.url.as_bytes())
+                        .iter()
+                        .map(ToString::to_string)
+                        .collect();
+
+                    let plugin_directory = ZELLIJ_CACHE_DIR.join(hash);
+
+                    // The plugin path is determined by the hash of the plugin URL in the cache directory.
+                    plugin.path = plugin_directory.join(&download.file_name);
+
+                    let downloader = Downloader::new(plugin_directory);
+                    match downloader.fetch(&download).await {
+                        Ok(_) => {},
+                        Err(e) => handle_plugin_loading_failure(
+                            &senders,
+                            plugin_id,
+                            &mut loading_indication,
+                            e,
+                        ),
+                    }
+                }
+
                 match PluginLoader::start_plugin(
                     plugin_id,
                     client_id,
@@ -342,6 +375,7 @@ impl WasmBridge {
         pid: PluginId,
         new_columns: usize,
         new_rows: usize,
+        shutdown_sender: Sender<()>,
     ) -> Result<()> {
         let err_context = move || format!("failed to resize plugin {pid}");
 
@@ -369,22 +403,29 @@ impl WasmBridge {
                     let running_plugin = running_plugin.clone();
                     let plugin_id = plugin_id;
                     let client_id = client_id;
+                    let _s = shutdown_sender.clone();
                     async move {
                         let mut running_plugin = running_plugin.lock().unwrap();
+                        let _s = _s; // guard to allow the task to complete before cleanup/shutdown
                         if running_plugin.apply_event_id(AtomicEvent::Resize, event_id) {
                             running_plugin.rows = new_rows;
                             running_plugin.columns = new_columns;
+
                             let rendered_bytes = running_plugin
                                 .instance
+                                .clone()
                                 .exports
                                 .get_function("render")
                                 .map_err(anyError::new)
                                 .and_then(|render| {
                                     render
-                                        .call(&[
-                                            Value::I32(running_plugin.rows as i32),
-                                            Value::I32(running_plugin.columns as i32),
-                                        ])
+                                        .call(
+                                            &mut running_plugin.store,
+                                            &[
+                                                Value::I32(new_rows as i32),
+                                                Value::I32(new_columns as i32),
+                                            ],
+                                        )
                                         .map_err(anyError::new)
                                 })
                                 .and_then(|_| wasi_read_string(&running_plugin.plugin_env.wasi_env))
@@ -420,6 +461,7 @@ impl WasmBridge {
     pub fn update_plugins(
         &mut self,
         mut updates: Vec<(Option<PluginId>, Option<ClientId>, Event)>,
+        shutdown_sender: Sender<()>,
     ) -> Result<()> {
         let err_context = || "failed to update plugin state".to_string();
 
@@ -459,17 +501,16 @@ impl WasmBridge {
                         let event = event.clone();
                         let plugin_id = *plugin_id;
                         let client_id = *client_id;
+                        let _s = shutdown_sender.clone();
                         async move {
-                            let running_plugin = running_plugin.lock().unwrap();
+                            let mut running_plugin = running_plugin.lock().unwrap();
                             let mut plugin_bytes = vec![];
+                            let _s = _s; // guard to allow the task to complete before cleanup/shutdown
                             match apply_event_to_plugin(
                                 plugin_id,
                                 client_id,
-                                &running_plugin.instance,
-                                &running_plugin.plugin_env,
+                                &mut running_plugin,
                                 &event,
-                                running_plugin.rows,
-                                running_plugin.columns,
                                 &mut plugin_bytes,
                             ) {
                                 Ok(()) => {
@@ -503,11 +544,15 @@ impl WasmBridge {
         }
         Ok(())
     }
-    pub fn apply_cached_events(&mut self, plugin_ids: Vec<PluginId>) -> Result<()> {
+    pub fn apply_cached_events(
+        &mut self,
+        plugin_ids: Vec<PluginId>,
+        shutdown_sender: Sender<()>,
+    ) -> Result<()> {
         let mut applied_plugin_paths = HashSet::new();
         for plugin_id in plugin_ids {
-            self.apply_cached_events_and_resizes_for_plugin(plugin_id)?;
-            if let Some(run_plugin) = self.run_plugin_of_plugin_id(plugin_id) {
+            self.apply_cached_events_and_resizes_for_plugin(plugin_id, shutdown_sender.clone())?;
+            if let Some(run_plugin) = self.run_plugin_of_loading_plugin_id(plugin_id) {
                 applied_plugin_paths.insert(run_plugin.clone());
             }
             self.loading_plugins
@@ -538,13 +583,23 @@ impl WasmBridge {
             watcher.stop_nonblocking();
         }
     }
-    fn run_plugin_of_plugin_id(&self, plugin_id: PluginId) -> Option<&RunPlugin> {
+    pub fn run_plugin_of_loading_plugin_id(&self, plugin_id: PluginId) -> Option<&RunPlugin> {
         self.loading_plugins
             .iter()
             .find(|((p_id, _run_plugin), _)| p_id == &plugin_id)
             .map(|((_p_id, run_plugin), _)| run_plugin)
     }
-    fn apply_cached_events_and_resizes_for_plugin(&mut self, plugin_id: PluginId) -> Result<()> {
+    pub fn run_plugin_of_plugin_id(&self, plugin_id: PluginId) -> Option<RunPlugin> {
+        self.plugin_map
+            .lock()
+            .unwrap()
+            .run_plugin_of_plugin_id(plugin_id)
+    }
+    fn apply_cached_events_and_resizes_for_plugin(
+        &mut self,
+        plugin_id: PluginId,
+        shutdown_sender: Sender<()>,
+    ) -> Result<()> {
         let err_context = || format!("Failed to apply cached events to plugin");
         if let Some(events) = self.cached_events_for_pending_plugins.remove(&plugin_id) {
             let all_connected_clients: Vec<ClientId> = self
@@ -572,17 +627,16 @@ impl WasmBridge {
                             let senders = self.senders.clone();
                             let running_plugin = running_plugin.clone();
                             let client_id = *client_id;
+                            let _s = shutdown_sender.clone();
                             async move {
-                                let running_plugin = running_plugin.lock().unwrap();
+                                let mut running_plugin = running_plugin.lock().unwrap();
                                 let mut plugin_bytes = vec![];
+                                let _s = _s; // guard to allow the task to complete before cleanup/shutdown
                                 match apply_event_to_plugin(
                                     plugin_id,
                                     client_id,
-                                    &running_plugin.instance,
-                                    &running_plugin.plugin_env,
+                                    &mut running_plugin,
                                     &event,
-                                    running_plugin.rows,
-                                    running_plugin.columns,
                                     &mut plugin_bytes,
                                 ) {
                                     Ok(()) => {
@@ -601,7 +655,7 @@ impl WasmBridge {
             }
         }
         if let Some((rows, columns)) = self.cached_resizes_for_pending_plugins.remove(&plugin_id) {
-            self.resize_plugin(plugin_id, columns, rows)?;
+            self.resize_plugin(plugin_id, columns, rows, shutdown_sender.clone())?;
         }
         self.apply_cached_worker_messages(plugin_id)?;
         Ok(())
@@ -716,35 +770,34 @@ impl WasmBridge {
         status: PermissionStatus,
         cache_path: Option<PathBuf>,
     ) -> Result<()> {
-        if let Some(running_plugin) = self
+        let err_context = || format!("Failed to write plugin permission {plugin_id}");
+
+        let running_plugin = self
             .plugin_map
             .lock()
             .unwrap()
             .get_running_plugin(plugin_id, client_id)
-        {
-            let err_context = || format!("Failed to write plugin permission {plugin_id}");
+            .ok_or_else(|| anyhow!("Failed to get running plugin"))?;
 
-            let mut running_plugin = running_plugin.lock().unwrap();
-            let permissions = if status == PermissionStatus::Granted {
-                permissions
-            } else {
-                vec![]
-            };
+        let mut running_plugin = running_plugin.lock().unwrap();
 
-            running_plugin
-                .plugin_env
-                .set_permissions(HashSet::from_iter(permissions.clone()));
+        let permissions = if status == PermissionStatus::Granted {
+            permissions
+        } else {
+            vec![]
+        };
 
-            let mut permission_cache = PermissionCache::from_path_or_default(cache_path);
-            permission_cache.cache(
-                running_plugin.plugin_env.plugin.location.to_string(),
-                permissions,
-            );
+        running_plugin
+            .plugin_env
+            .set_permissions(HashSet::from_iter(permissions.clone()));
 
-            permission_cache.write_to_file().with_context(err_context)?;
-        }
+        let mut permission_cache = PermissionCache::from_path_or_default(cache_path);
+        permission_cache.cache(
+            running_plugin.plugin_env.plugin.location.to_string(),
+            permissions,
+        );
 
-        Ok(())
+        permission_cache.write_to_file().with_context(err_context)
     }
 }
 
@@ -801,13 +854,15 @@ fn check_event_permission(
 pub fn apply_event_to_plugin(
     plugin_id: PluginId,
     client_id: ClientId,
-    instance: &Instance,
-    plugin_env: &PluginEnv,
+    running_plugin: &mut RunningPlugin,
     event: &Event,
-    rows: usize,
-    columns: usize,
     plugin_bytes: &mut Vec<(PluginId, ClientId, Vec<u8>)>,
 ) -> Result<()> {
+    let instance = &running_plugin.instance;
+    let plugin_env = &running_plugin.plugin_env;
+    let rows = running_plugin.rows;
+    let columns = running_plugin.columns;
+
     let err_context = || format!("Failed to apply event to plugin {plugin_id}");
     match check_event_permission(plugin_env, event) {
         (PermissionStatus::Granted, _) => {
@@ -821,7 +876,9 @@ pub fn apply_event_to_plugin(
                 .with_context(err_context)?;
             wasi_write_object(&plugin_env.wasi_env, &protobuf_event.encode_to_vec())
                 .with_context(err_context)?;
-            let update_return = update.call(&[]).with_context(err_context)?;
+            let update_return = update
+                .call(&mut running_plugin.store, &[])
+                .with_context(err_context)?;
             let mut should_render = match update_return.get(0) {
                 Some(Value::I32(n)) => *n == 1,
                 _ => false,
@@ -838,7 +895,10 @@ pub fn apply_event_to_plugin(
                     .map_err(anyError::new)
                     .and_then(|render| {
                         render
-                            .call(&[Value::I32(rows as i32), Value::I32(columns as i32)])
+                            .call(
+                                &mut running_plugin.store,
+                                &[Value::I32(rows as i32), Value::I32(columns as i32)],
+                            )
                             .map_err(anyError::new)
                     })
                     .and_then(|_| wasi_read_string(&plugin_env.wasi_env))
