@@ -1,4 +1,5 @@
 use super::PluginInstruction;
+use crate::background_jobs::BackgroundJob;
 use crate::plugins::plugin_map::{PluginEnv, Subscriptions};
 use crate::plugins::wasm_bridge::handle_plugin_crash;
 use crate::route::route_action;
@@ -14,10 +15,10 @@ use std::{
     thread,
     time::{Duration, Instant},
 };
-use wasmer::{imports, Function, ImportObject, Store, WasmerEnv};
+use wasmer::{imports, AsStoreMut, Function, FunctionEnv, FunctionEnvMut, Imports};
 use wasmer_wasi::WasiEnv;
 use zellij_utils::data::{
-    CommandType, ConnectToSession, PermissionStatus, PermissionType, PluginPermission,
+    CommandType, ConnectToSession, HttpVerb, PermissionStatus, PermissionType, PluginPermission,
 };
 use zellij_utils::input::permission::PermissionCache;
 
@@ -26,7 +27,7 @@ use url::Url;
 use crate::{panes::PaneId, screen::ScreenInstruction};
 
 use zellij_utils::{
-    consts::VERSION,
+    consts::{VERSION, ZELLIJ_SESSION_INFO_CACHE_DIR, ZELLIJ_SOCK_DIR},
     data::{
         CommandToRun, Direction, Event, EventType, FileToOpen, InputMode, PluginCommand, PluginIds,
         PluginMessage, Resize, ResizeStrategy,
@@ -51,6 +52,7 @@ macro_rules! apply_action {
         if let Err(e) = route_action(
             $action,
             $env.plugin_env.client_id,
+            Some(PaneId::Plugin($env.plugin_env.plugin_id)),
             $env.plugin_env.senders.clone(),
             $env.plugin_env.capabilities.clone(),
             $env.plugin_env.client_attributes.clone(),
@@ -63,20 +65,21 @@ macro_rules! apply_action {
 }
 
 pub fn zellij_exports(
-    store: &Store,
+    store: &mut impl AsStoreMut,
     plugin_env: &PluginEnv,
     subscriptions: &Arc<Mutex<Subscriptions>>,
-) -> ImportObject {
+) -> Imports {
+    let function_env = FunctionEnv::new(store, ForeignFunctionEnv::new(plugin_env, subscriptions));
     imports! {
         "zellij" => {
           "host_run_plugin_command" => {
-            Function::new_native_with_env(store, ForeignFunctionEnv::new(plugin_env, subscriptions), host_run_plugin_command)
+            Function::new_typed_with_env(store, &function_env, host_run_plugin_command)
           }
         }
     }
 }
 
-#[derive(WasmerEnv, Clone)]
+#[derive(Clone)]
 pub struct ForeignFunctionEnv {
     pub plugin_env: PluginEnv,
     pub subscriptions: Arc<Mutex<Subscriptions>>,
@@ -91,7 +94,8 @@ impl ForeignFunctionEnv {
     }
 }
 
-fn host_run_plugin_command(env: &ForeignFunctionEnv) {
+fn host_run_plugin_command(env: FunctionEnvMut<ForeignFunctionEnv>) {
+    let env = env.data();
     let err_context = || format!("failed to run plugin command {}", env.plugin_env.name());
     wasi_read_bytes(&env.plugin_env.wasi_env)
         .and_then(|bytes| {
@@ -123,6 +127,12 @@ fn host_run_plugin_command(env: &ForeignFunctionEnv) {
                     PluginCommand::SwitchTabTo(tab_index) => switch_tab_to(env, tab_index),
                     PluginCommand::SetTimeout(seconds) => set_timeout(env, seconds),
                     PluginCommand::ExecCmd(command_line) => exec_cmd(env, command_line),
+                    PluginCommand::RunCommand(command_line, env_variables, cwd, context) => {
+                        run_command(env, command_line, env_variables, cwd, context)
+                    },
+                    PluginCommand::WebRequest(url, verb, headers, body, context) => {
+                        web_request(env, url, verb, headers, body, context)
+                    },
                     PluginCommand::PostMessageTo(plugin_message) => {
                         post_message_to(env, plugin_message)?
                     },
@@ -214,6 +224,22 @@ fn host_run_plugin_command(env: &ForeignFunctionEnv) {
                         connect_to_session.tab_position,
                         connect_to_session.pane_id,
                     )?,
+                    PluginCommand::DeleteDeadSession(session_name) => {
+                        delete_dead_session(session_name)?
+                    },
+                    PluginCommand::DeleteAllDeadSessions => delete_all_dead_sessions()?,
+                    PluginCommand::OpenFileInPlace(file_to_open) => {
+                        open_file_in_place(env, file_to_open)
+                    },
+                    PluginCommand::OpenTerminalInPlace(cwd) => {
+                        open_terminal_in_place(env, cwd.path.try_into()?)
+                    },
+                    PluginCommand::OpenCommandPaneInPlace(command_to_run) => {
+                        open_command_pane_in_place(env, command_to_run)
+                    },
+                    PluginCommand::RenameSession(new_session_name) => {
+                        rename_session(env, new_session_name)
+                    },
                 },
                 (PermissionStatus::Denied, permission) => {
                     log::error!(
@@ -347,12 +373,14 @@ fn get_zellij_version(env: &ForeignFunctionEnv) {
 fn open_file(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
     let error_msg = || format!("failed to open file in plugin {}", env.plugin_env.name());
     let floating = false;
+    let in_place = false;
     let action = Action::EditFile(
         file_to_open.path,
         file_to_open.line_number,
         file_to_open.cwd,
         None,
         floating,
+        in_place,
     );
     apply_action!(action, error_msg, env);
 }
@@ -360,12 +388,29 @@ fn open_file(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
 fn open_file_floating(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
     let error_msg = || format!("failed to open file in plugin {}", env.plugin_env.name());
     let floating = true;
+    let in_place = false;
     let action = Action::EditFile(
         file_to_open.path,
         file_to_open.line_number,
         file_to_open.cwd,
         None,
         floating,
+        in_place,
+    );
+    apply_action!(action, error_msg, env);
+}
+
+fn open_file_in_place(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
+    let error_msg = || format!("failed to open file in plugin {}", env.plugin_env.name());
+    let floating = false;
+    let in_place = true;
+    let action = Action::EditFile(
+        file_to_open.path,
+        file_to_open.line_number,
+        file_to_open.cwd,
+        None,
+        floating,
+        in_place,
     );
     apply_action!(action, error_msg, env);
 }
@@ -402,6 +447,22 @@ fn open_terminal_floating(env: &ForeignFunctionEnv, cwd: PathBuf) {
     apply_action!(action, error_msg, env);
 }
 
+fn open_terminal_in_place(env: &ForeignFunctionEnv, cwd: PathBuf) {
+    let error_msg = || format!("failed to open file in plugin {}", env.plugin_env.name());
+    let mut default_shell = env
+        .plugin_env
+        .default_shell
+        .clone()
+        .unwrap_or_else(|| TerminalAction::RunCommand(RunCommand::default()));
+    default_shell.change_cwd(cwd);
+    let run_command_action: Option<RunCommandAction> = match default_shell {
+        TerminalAction::RunCommand(run_command) => Some(run_command.into()),
+        _ => None,
+    };
+    let action = Action::NewInPlacePane(run_command_action, None);
+    apply_action!(action, error_msg, env);
+}
+
 fn open_command_pane(env: &ForeignFunctionEnv, command_to_run: CommandToRun) {
     let error_msg = || format!("failed to open command in plugin {}", env.plugin_env.name());
     let command = command_to_run.path;
@@ -444,6 +505,27 @@ fn open_command_pane_floating(env: &ForeignFunctionEnv, command_to_run: CommandT
     apply_action!(action, error_msg, env);
 }
 
+fn open_command_pane_in_place(env: &ForeignFunctionEnv, command_to_run: CommandToRun) {
+    let error_msg = || format!("failed to open command in plugin {}", env.plugin_env.name());
+    let command = command_to_run.path;
+    let cwd = command_to_run.cwd;
+    let args = command_to_run.args;
+    let direction = None;
+    let hold_on_close = true;
+    let hold_on_start = false;
+    let name = None;
+    let run_command_action = RunCommandAction {
+        command,
+        args,
+        cwd,
+        direction,
+        hold_on_close,
+        hold_on_start,
+    };
+    let action = Action::NewInPlacePane(Some(run_command_action), name);
+    apply_action!(action, error_msg, env);
+}
+
 fn switch_tab_to(env: &ForeignFunctionEnv, tab_idx: u32) {
     env.plugin_env
         .senders
@@ -504,6 +586,7 @@ fn set_timeout(env: &ForeignFunctionEnv, secs: f64) {
 }
 
 fn exec_cmd(env: &ForeignFunctionEnv, mut command_line: Vec<String>) {
+    log::warn!("The ExecCmd plugin command is deprecated and will be removed in a future version. Please use RunCmd instead (it has all the things and can even show you STDOUT/STDERR and an exit code!)");
     let err_context = || {
         format!(
             "failed to execute command on host for plugin '{}'",
@@ -527,6 +610,54 @@ fn exec_cmd(env: &ForeignFunctionEnv, mut command_line: Vec<String>) {
         .non_fatal();
 }
 
+fn run_command(
+    env: &ForeignFunctionEnv,
+    mut command_line: Vec<String>,
+    env_variables: BTreeMap<String, String>,
+    cwd: PathBuf,
+    context: BTreeMap<String, String>,
+) {
+    if command_line.is_empty() {
+        log::error!("Command cannot be empty");
+    } else {
+        let command = command_line.remove(0);
+        let _ = env
+            .plugin_env
+            .senders
+            .send_to_background_jobs(BackgroundJob::RunCommand(
+                env.plugin_env.plugin_id,
+                env.plugin_env.client_id,
+                command,
+                command_line,
+                env_variables,
+                cwd,
+                context,
+            ));
+    }
+}
+
+fn web_request(
+    env: &ForeignFunctionEnv,
+    url: String,
+    verb: HttpVerb,
+    headers: BTreeMap<String, String>,
+    body: Vec<u8>,
+    context: BTreeMap<String, String>,
+) {
+    let _ = env
+        .plugin_env
+        .senders
+        .send_to_background_jobs(BackgroundJob::WebRequest(
+            env.plugin_env.plugin_id,
+            env.plugin_env.client_id,
+            url,
+            verb,
+            headers,
+            body,
+            context,
+        ));
+}
+
 fn post_message_to(env: &ForeignFunctionEnv, plugin_message: PluginMessage) -> Result<()> {
     let worker_name = plugin_message
         .worker_name
@@ -713,6 +844,52 @@ fn switch_session(
     Ok(())
 }
 
+fn delete_dead_session(session_name: String) -> Result<()> {
+    std::fs::remove_dir_all(&*ZELLIJ_SESSION_INFO_CACHE_DIR.join(&session_name))
+        .with_context(|| format!("Failed to delete dead session: {:?}", &session_name))
+}
+
+fn delete_all_dead_sessions() -> Result<()> {
+    use std::os::unix::fs::FileTypeExt;
+    let mut live_sessions = vec![];
+    if let Ok(files) = std::fs::read_dir(&*ZELLIJ_SOCK_DIR) {
+        files.for_each(|file| {
+            if let Ok(file) = file {
+                if let Ok(file_name) = file.file_name().into_string() {
+                    if file.file_type().unwrap().is_socket() {
+                        live_sessions.push(file_name);
+                    }
+                }
+            }
+        });
+    }
+    let dead_sessions: Vec<String> = match std::fs::read_dir(&*ZELLIJ_SESSION_INFO_CACHE_DIR) {
+        Ok(files_in_session_info_folder) => {
+            let files_that_are_folders = files_in_session_info_folder
+                .filter_map(|f| f.ok().map(|f| f.path()))
+                .filter(|f| f.is_dir());
+            files_that_are_folders
+                .filter_map(|folder_name| {
+                    let session_name = folder_name.file_name()?.to_str()?.to_owned();
+                    if live_sessions.contains(&session_name) {
+                        // this is not a dead session...
+                        return None;
+                    }
+                    Some(session_name)
+                })
+                .collect()
+        },
+        Err(e) => {
+            log::error!("Failed to read session info cache dir: {:?}", e);
+            vec![]
+        },
+    };
+    for session in dead_sessions {
+        delete_dead_session(session)?;
+    }
+    Ok(())
+}
+
 fn edit_scrollback(env: &ForeignFunctionEnv) {
     let action = Action::EditScrollback;
     let error_msg = || format!("Failed to edit scrollback");
@@ -1014,6 +1191,17 @@ fn rename_tab(env: &ForeignFunctionEnv, tab_index: u32, new_name: &str) {
     apply_action!(rename_tab_action, error_msg, env);
 }
 
+fn rename_session(env: &ForeignFunctionEnv, new_session_name: String) {
+    let error_msg = || {
+        format!(
+            "failed to rename session in plugin {}",
+            env.plugin_env.name()
+        )
+    };
+    let action = Action::RenameSession(new_session_name);
+    apply_action!(action, error_msg, env);
+}
+
 // Custom panic handler for plugins.
 //
 // This is called when a panic occurs in a plugin. Since most panics will likely originate in the
@@ -1036,17 +1224,13 @@ pub fn wasi_read_string(wasi_env: &WasiEnv) -> Result<String> {
     let mut buf = vec![];
     wasi_env
         .state()
-        .fs
-        .stdout_mut()
+        .stdout()
         .map_err(anyError::new)
-        .and_then(|stdout| {
-            stdout
-                .as_mut()
-                .ok_or(anyhow!("failed to get mutable reference to stdout"))
-        })
-        .and_then(|wasi_file| wasi_file.read_to_end(&mut buf).map_err(anyError::new))
+        .and_then(|stdout| stdout.ok_or(anyhow!("failed to get mutable reference to stdout")))
+        .and_then(|mut wasi_file| wasi_file.read_to_end(&mut buf).map_err(anyError::new))
         .with_context(err_context)?;
     let buf = String::from_utf8_lossy(&buf);
+
     // https://stackoverflow.com/questions/66450942/in-rust-is-there-a-way-to-make-literal-newlines-in-r-using-windows-c
     Ok(buf.replace("\n", "\n\r"))
 }
@@ -1054,15 +1238,10 @@ pub fn wasi_read_string(wasi_env: &WasiEnv) -> Result<String> {
 pub fn wasi_write_string(wasi_env: &WasiEnv, buf: &str) -> Result<()> {
     wasi_env
         .state()
-        .fs
-        .stdin_mut()
+        .stdin()
         .map_err(anyError::new)
-        .and_then(|stdin| {
-            stdin
-                .as_mut()
-                .ok_or(anyhow!("failed to get mutable reference to stdin"))
-        })
-        .and_then(|stdin| writeln!(stdin, "{}\r", buf).map_err(anyError::new))
+        .and_then(|stdin| stdin.ok_or(anyhow!("failed to get mutable reference to stdin")))
+        .and_then(|mut stdin| writeln!(stdin, "{}\r", buf).map_err(anyError::new))
         .with_context(|| format!("failed to write string to WASI env '{wasi_env:?}'"))
 }
 
@@ -1090,15 +1269,19 @@ fn check_command_permission(
         return (PermissionStatus::Granted, None);
     }
     let permission = match command {
-        PluginCommand::OpenFile(..) | PluginCommand::OpenFileFloating(..) => {
-            PermissionType::OpenFiles
-        },
+        PluginCommand::OpenFile(..)
+        | PluginCommand::OpenFileFloating(..)
+        | PluginCommand::OpenFileInPlace(..) => PermissionType::OpenFiles,
         PluginCommand::OpenTerminal(..)
         | PluginCommand::StartOrReloadPlugin(..)
-        | PluginCommand::OpenTerminalFloating(..) => PermissionType::OpenTerminalsOrPlugins,
+        | PluginCommand::OpenTerminalFloating(..)
+        | PluginCommand::OpenTerminalInPlace(..) => PermissionType::OpenTerminalsOrPlugins,
         PluginCommand::OpenCommandPane(..)
         | PluginCommand::OpenCommandPaneFloating(..)
+        | PluginCommand::OpenCommandPaneInPlace(..)
+        | PluginCommand::RunCommand(..)
         | PluginCommand::ExecCmd(..) => PermissionType::RunCommands,
+        PluginCommand::WebRequest(..) => PermissionType::WebAccess,
         PluginCommand::Write(..) | PluginCommand::WriteChars(..) => PermissionType::WriteToStdin,
         PluginCommand::SwitchTabTo(..)
         | PluginCommand::SwitchToMode(..)
@@ -1144,6 +1327,9 @@ fn check_command_permission(
         | PluginCommand::RenameTerminalPane(..)
         | PluginCommand::RenamePluginPane(..)
         | PluginCommand::SwitchSession(..)
+        | PluginCommand::DeleteDeadSession(..)
+        | PluginCommand::DeleteAllDeadSessions
+        | PluginCommand::RenameSession(..)
         | PluginCommand::RenameTab(..) => PermissionType::ChangeApplicationState,
         _ => return (PermissionStatus::Granted, None),
     };
