@@ -1,8 +1,10 @@
+use crate::background_jobs::BackgroundJob;
 use crate::terminal_bytes::TerminalBytes;
 use crate::{
     panes::PaneId,
     plugins::PluginInstruction,
     screen::ScreenInstruction,
+    session_layout_metadata::SessionLayoutMetadata,
     thread_bus::{Bus, ThreadSenders},
     ClientId, ServerInstruction,
 };
@@ -16,19 +18,22 @@ use zellij_utils::{
     input::{
         command::{RunCommand, TerminalAction},
         layout::{
-            FloatingPaneLayout, Layout, PluginUserConfiguration, Run, RunPluginLocation,
+            FloatingPaneLayout, Layout, PluginUserConfiguration, Run, RunPlugin, RunPluginLocation,
             TiledPaneLayout,
         },
     },
+    pane_size::Size,
+    session_serialization,
 };
 
 pub type VteBytes = Vec<u8>;
 pub type TabIndex = u32;
 
 #[derive(Clone, Copy, Debug)]
-pub enum ClientOrTabIndex {
+pub enum ClientTabIndexOrPaneId {
     ClientId(ClientId),
     TabIndex(usize),
+    PaneId(PaneId),
 }
 
 /// Instructions related to PTYs (pseudoterminals).
@@ -38,7 +43,7 @@ pub enum PtyInstruction {
         Option<TerminalAction>,
         Option<bool>,
         Option<String>,
-        ClientOrTabIndex,
+        ClientTabIndexOrPaneId,
     ), // bool (if Some) is
     // should_float, String is an optional pane name
     OpenInPlaceEditor(PathBuf, Option<usize>, ClientId), // Option<usize> is the optional line number
@@ -62,6 +67,24 @@ pub enum PtyInstruction {
     ClosePane(PaneId),
     CloseTab(Vec<PaneId>),
     ReRunCommandInPane(PaneId, RunCommand),
+    DropToShellInPane(PaneId, Option<PathBuf>), // Option<PathBuf> - default shell
+    SpawnInPlaceTerminal(
+        Option<TerminalAction>,
+        Option<String>,
+        ClientTabIndexOrPaneId,
+    ), // String is an optional pane name
+    DumpLayout(SessionLayoutMetadata, ClientId),
+    LogLayoutToHd(SessionLayoutMetadata),
+    FillPluginCwd(
+        Option<bool>,   // should float
+        bool,           // should be opened in place
+        Option<String>, // pane title
+        RunPlugin,
+        usize,          // tab index
+        Option<PaneId>, // pane id to replace if this is to be opened "in-place"
+        ClientId,
+        Size,
+    ),
     Exit,
 }
 
@@ -78,6 +101,11 @@ impl From<&PtyInstruction> for PtyContext {
             PtyInstruction::CloseTab(_) => PtyContext::CloseTab,
             PtyInstruction::NewTab(..) => PtyContext::NewTab,
             PtyInstruction::ReRunCommandInPane(..) => PtyContext::ReRunCommandInPane,
+            PtyInstruction::DropToShellInPane(..) => PtyContext::DropToShellInPane,
+            PtyInstruction::SpawnInPlaceTerminal(..) => PtyContext::SpawnInPlaceTerminal,
+            PtyInstruction::DumpLayout(..) => PtyContext::DumpLayout,
+            PtyInstruction::LogLayoutToHd(..) => PtyContext::LogLayoutToHd,
+            PtyInstruction::FillPluginCwd(..) => PtyContext::FillPluginCwd,
             PtyInstruction::Exit => PtyContext::Exit,
         }
     }
@@ -114,12 +142,26 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     ),
                     _ => (false, None, name),
                 };
+                let invoked_with =
+                    match &terminal_action {
+                        Some(TerminalAction::RunCommand(run_command)) => {
+                            Some(Run::Command(run_command.clone()))
+                        },
+                        Some(TerminalAction::OpenFile(file, line_number, cwd)) => Some(
+                            Run::EditFile(file.clone(), line_number.clone(), cwd.clone()),
+                        ),
+                        _ => None,
+                    };
                 match pty
                     .spawn_terminal(terminal_action, client_or_tab_index)
                     .with_context(err_context)
                 {
                     Ok((pid, starts_held)) => {
-                        let hold_for_command = if starts_held { run_command } else { None };
+                        let hold_for_command = if starts_held {
+                            run_command.clone()
+                        } else {
+                            None
+                        };
                         pty.bus
                             .senders
                             .send_to_screen(ScreenInstruction::NewPane(
@@ -127,6 +169,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                 pane_title,
                                 should_float,
                                 hold_for_command,
+                                invoked_with,
                                 client_or_tab_index,
                             ))
                             .with_context(err_context)?;
@@ -142,6 +185,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                         pane_title,
                                         should_float,
                                         hold_for_command,
+                                        invoked_with,
                                         client_or_tab_index,
                                     ))
                                     .with_context(err_context)?;
@@ -164,13 +208,92 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     },
                 }
             },
+            PtyInstruction::SpawnInPlaceTerminal(
+                terminal_action,
+                name,
+                client_id_tab_index_or_pane_id,
+            ) => {
+                let err_context = || {
+                    format!(
+                        "failed to spawn terminal for {:?}",
+                        client_id_tab_index_or_pane_id
+                    )
+                };
+                let (hold_on_close, run_command, pane_title) = match &terminal_action {
+                    Some(TerminalAction::RunCommand(run_command)) => (
+                        run_command.hold_on_close,
+                        Some(run_command.clone()),
+                        Some(name.unwrap_or_else(|| run_command.to_string())),
+                    ),
+                    _ => (false, None, name),
+                };
+                let invoked_with =
+                    match &terminal_action {
+                        Some(TerminalAction::RunCommand(run_command)) => {
+                            Some(Run::Command(run_command.clone()))
+                        },
+                        Some(TerminalAction::OpenFile(file, line_number, cwd)) => Some(
+                            Run::EditFile(file.clone(), line_number.clone(), cwd.clone()),
+                        ),
+                        _ => None,
+                    };
+                match pty
+                    .spawn_terminal(terminal_action, client_id_tab_index_or_pane_id)
+                    .with_context(err_context)
+                {
+                    Ok((pid, starts_held)) => {
+                        let hold_for_command = if starts_held { run_command } else { None };
+                        pty.bus
+                            .senders
+                            .send_to_screen(ScreenInstruction::ReplacePane(
+                                PaneId::Terminal(pid),
+                                hold_for_command,
+                                pane_title,
+                                invoked_with,
+                                client_id_tab_index_or_pane_id,
+                            ))
+                            .with_context(err_context)?;
+                    },
+                    Err(err) => match err.downcast_ref::<ZellijError>() {
+                        Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
+                            if hold_on_close {
+                                let hold_for_command = None; // we do not hold an "error" pane
+                                pty.bus
+                                    .senders
+                                    .send_to_screen(ScreenInstruction::ReplacePane(
+                                        PaneId::Terminal(*terminal_id),
+                                        hold_for_command,
+                                        pane_title,
+                                        invoked_with,
+                                        client_id_tab_index_or_pane_id,
+                                    ))
+                                    .with_context(err_context)?;
+                                if let Some(run_command) = run_command {
+                                    send_command_not_found_to_screen(
+                                        pty.bus.senders.clone(),
+                                        *terminal_id,
+                                        run_command.clone(),
+                                        None,
+                                    )
+                                    .with_context(err_context)?;
+                                }
+                            } else {
+                                log::error!("Failed to spawn terminal: {:?}", err);
+                                pty.close_pane(PaneId::Terminal(*terminal_id))
+                                    .with_context(err_context)?;
+                            }
+                        },
+                        _ => Err::<(), _>(err).non_fatal(),
+                    },
+                }
+            },
             PtyInstruction::OpenInPlaceEditor(temp_file, line_number, client_id) => {
                 let err_context =
                     || format!("failed to open in-place editor for client {}", client_id);
 
                 match pty.spawn_terminal(
                     Some(TerminalAction::OpenFile(temp_file, line_number, None)),
-                    ClientOrTabIndex::ClientId(client_id),
+                    ClientTabIndexOrPaneId::ClientId(client_id),
                 ) {
                     Ok((pid, _starts_held)) => {
                         pty.bus
@@ -199,7 +322,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     _ => (false, None, name),
                 };
                 match pty
-                    .spawn_terminal(terminal_action, ClientOrTabIndex::ClientId(client_id))
+                    .spawn_terminal(terminal_action, ClientTabIndexOrPaneId::ClientId(client_id))
                     .with_context(err_context)
                 {
                     Ok((pid, starts_held)) => {
@@ -270,7 +393,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     _ => (false, None, name),
                 };
                 match pty
-                    .spawn_terminal(terminal_action, ClientOrTabIndex::ClientId(client_id))
+                    .spawn_terminal(terminal_action, ClientTabIndexOrPaneId::ClientId(client_id))
                     .with_context(err_context)
                 {
                     Ok((pid, starts_held)) => {
@@ -423,6 +546,117 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     },
                 }
             },
+            PtyInstruction::DropToShellInPane(pane_id, default_shell) => {
+                let err_context = || format!("failed to rerun command in pane {:?}", pane_id);
+
+                // TODO: get configured default_shell from screen/tab as an option and default to
+                // this otherwise (also look for a place that turns get_default_shell into a
+                // RunCommand, we might have done this before)
+                let run_command = RunCommand {
+                    command: default_shell.unwrap_or_else(|| get_default_shell()),
+                    hold_on_close: false,
+                    hold_on_start: false,
+                    // TODO: cwd
+                    ..Default::default()
+                };
+                match pty
+                    .rerun_command_in_pane(pane_id, run_command.clone())
+                    .with_context(err_context)
+                {
+                    Ok(..) => {},
+                    Err(err) => match err.downcast_ref::<ZellijError>() {
+                        Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
+                            if run_command.hold_on_close {
+                                pty.bus
+                                    .senders
+                                    .send_to_screen(ScreenInstruction::PtyBytes(
+                                        *terminal_id,
+                                        format!(
+                                            "Command not found: {}",
+                                            run_command.command.display()
+                                        )
+                                        .as_bytes()
+                                        .to_vec(),
+                                    ))
+                                    .with_context(err_context)?;
+                                pty.bus
+                                    .senders
+                                    .send_to_screen(ScreenInstruction::HoldPane(
+                                        PaneId::Terminal(*terminal_id),
+                                        Some(2), // exit status
+                                        run_command,
+                                        None,
+                                        None,
+                                    ))
+                                    .with_context(err_context)?;
+                            }
+                        },
+                        _ => Err::<(), _>(err).non_fatal(),
+                    },
+                }
+            },
+            PtyInstruction::DumpLayout(mut session_layout_metadata, client_id) => {
+                let err_context = || format!("Failed to dump layout");
+                pty.populate_session_layout_metadata(&mut session_layout_metadata);
+                match session_serialization::serialize_session_layout(
+                    session_layout_metadata.into(),
+                ) {
+                    Ok((kdl_layout, _pane_contents)) => {
+                        pty.bus
+                            .senders
+                            .send_to_server(ServerInstruction::Log(vec![kdl_layout], client_id))
+                            .with_context(err_context)
+                            .non_fatal();
+                    },
+                    Err(e) => {
+                        pty.bus
+                            .senders
+                            .send_to_server(ServerInstruction::Log(vec![e.to_owned()], client_id))
+                            .with_context(err_context)
+                            .non_fatal();
+                    },
+                }
+            },
+            PtyInstruction::LogLayoutToHd(mut session_layout_metadata) => {
+                let err_context = || format!("Failed to dump layout");
+                pty.populate_session_layout_metadata(&mut session_layout_metadata);
+                match session_serialization::serialize_session_layout(
+                    session_layout_metadata.into(),
+                ) {
+                    Ok(kdl_layout_and_pane_contents) => {
+                        pty.bus
+                            .senders
+                            .send_to_background_jobs(BackgroundJob::ReportLayoutInfo(
+                                kdl_layout_and_pane_contents,
+                            ))
+                            .with_context(err_context)?;
+                    },
+                    Err(e) => {
+                        log::error!("Failed to log layout to HD: {}", e);
+                    },
+                }
+            },
+            PtyInstruction::FillPluginCwd(
+                should_float,
+                should_be_open_in_place,
+                pane_title,
+                run,
+                tab_index,
+                pane_id_to_replace,
+                client_id,
+                size,
+            ) => {
+                pty.fill_plugin_cwd(
+                    should_float,
+                    should_be_open_in_place,
+                    pane_title,
+                    run,
+                    tab_index,
+                    pane_id_to_replace,
+                    client_id,
+                    size,
+                )?;
+            },
             PtyInstruction::Exit => break,
         }
     }
@@ -501,25 +735,45 @@ impl Pty {
             };
         };
     }
+    fn fill_cwd_from_pane_id(&self, terminal_action: &mut TerminalAction, pane_id: &u32) {
+        if let TerminalAction::RunCommand(run_command) = terminal_action {
+            if run_command.cwd.is_none() {
+                run_command.cwd = self.id_to_child_pid.get(pane_id).and_then(|&id| {
+                    self.bus
+                        .os_input
+                        .as_ref()
+                        .and_then(|input| input.get_cwd(Pid::from_raw(id)))
+                });
+            };
+        };
+    }
     pub fn spawn_terminal(
         &mut self,
         terminal_action: Option<TerminalAction>,
-        client_or_tab_index: ClientOrTabIndex,
+        client_or_tab_index: ClientTabIndexOrPaneId,
     ) -> Result<(u32, bool)> {
         // bool is starts_held
         let err_context = || format!("failed to spawn terminal for {:?}", client_or_tab_index);
 
         // returns the terminal id
         let terminal_action = match client_or_tab_index {
-            ClientOrTabIndex::ClientId(client_id) => {
+            ClientTabIndexOrPaneId::ClientId(client_id) => {
                 let mut terminal_action =
                     terminal_action.unwrap_or_else(|| self.get_default_terminal(None, None));
                 self.fill_cwd(&mut terminal_action, client_id);
                 terminal_action
             },
-            ClientOrTabIndex::TabIndex(_) => {
+            ClientTabIndexOrPaneId::TabIndex(_) => {
                 terminal_action.unwrap_or_else(|| self.get_default_terminal(None, None))
             },
+            ClientTabIndexOrPaneId::PaneId(pane_id) => {
+                let mut terminal_action =
+                    terminal_action.unwrap_or_else(|| self.get_default_terminal(None, None));
+                if let PaneId::Terminal(terminal_pane_id) = pane_id {
+                    self.fill_cwd_from_pane_id(&mut terminal_action, &terminal_pane_id);
+                }
+                terminal_action
+            },
         };
         let (hold_on_start, hold_on_close) = match &terminal_action {
             TerminalAction::RunCommand(run_command) => {
@@ -1011,6 +1265,89 @@ impl Pty {
             _ => Err(anyhow!("cannot respawn plugin panes")).with_context(err_context),
         }
     }
+    pub fn populate_session_layout_metadata(
+        &self,
+        session_layout_metadata: &mut SessionLayoutMetadata,
+    ) {
+        let terminal_ids = session_layout_metadata.all_terminal_ids();
+        let mut terminal_ids_to_commands: HashMap<u32, Vec<String>> = HashMap::new();
+        let mut terminal_ids_to_cwds: HashMap<u32, PathBuf> = HashMap::new();
+
+        let pids: Vec<_> = terminal_ids
+            .iter()
+            .filter_map(|id| self.id_to_child_pid.get(&id))
+            .map(|pid| Pid::from_raw(*pid))
+            .collect();
+        let pids_to_cwds = self
+            .bus
+            .os_input
+            .as_ref()
+            .map(|os_input| os_input.get_cwds(pids))
+            .unwrap_or_default();
+        let ppids_to_cmds = self
+            .bus
+            .os_input
+            .as_ref()
+            .map(|os_input| os_input.get_all_cmds_by_ppid())
+            .unwrap_or_default();
+
+        for terminal_id in terminal_ids {
+            let process_id = self.id_to_child_pid.get(&terminal_id);
+            let cwd = process_id
+                .as_ref()
+                .and_then(|pid| pids_to_cwds.get(&Pid::from_raw(**pid)));
+            let cmd = process_id
+                .as_ref()
+                .and_then(|pid| ppids_to_cmds.get(&format!("{}", pid)));
+            if let Some(cmd) = cmd {
+                terminal_ids_to_commands.insert(terminal_id, cmd.clone());
+            }
+            if let Some(cwd) = cwd {
+                terminal_ids_to_cwds.insert(terminal_id, cwd.clone());
+            }
+        }
+        session_layout_metadata.update_default_shell(get_default_shell());
+        session_layout_metadata.update_terminal_commands(terminal_ids_to_commands);
+        session_layout_metadata.update_terminal_cwds(terminal_ids_to_cwds);
+    }
+    pub fn fill_plugin_cwd(
+        &self,
+        should_float: Option<bool>,
+        should_open_in_place: bool, // should be opened in place
+        pane_title: Option<String>, // pane title
+        run: RunPlugin,
+        tab_index: usize,                   // tab index
+        pane_id_to_replace: Option<PaneId>, // pane id to replace if this is to be opened "in-place"
+        client_id: ClientId,
+        size: Size,
+    ) -> Result<()> {
+        let cwd = self
+            .active_panes
+            .get(&client_id)
+            .and_then(|pane| match pane {
+                PaneId::Plugin(..) => None,
+                PaneId::Terminal(id) => self.id_to_child_pid.get(id),
+            })
+            .and_then(|&id| {
+                self.bus
+                    .os_input
+                    .as_ref()
+                    .and_then(|input| input.get_cwd(Pid::from_raw(id)))
+            });
+
+        self.bus.senders.send_to_plugin(PluginInstruction::Load(
+            should_float,
+            should_open_in_place,
+            pane_title,
+            run,
+            tab_index,
+            pane_id_to_replace,
+            client_id,
+            size,
+            cwd,
+        ))?;
+        Ok(())
+    }
 }
 
 impl Drop for Pty {
