@@ -257,6 +257,7 @@ pub enum ScreenContext {
     Exit,
     ClearScreen,
     DumpScreen,
+    DumpLayout,
     EditScrollback,
     ScrollUp,
     ScrollUpAt,
@@ -343,6 +344,10 @@ pub enum ScreenContext {
     BreakPaneRight,
     BreakPaneLeft,
     UpdateSessionInfos,
+    ReplacePane,
+    NewInPlacePluginPane,
+    DumpLayoutToHd,
+    RenameSession,
 }
 
 /// Stack call representations corresponding to the different types of [`PtyInstruction`]s.
@@ -358,6 +363,11 @@ pub enum PtyContext {
     ClosePane,
     CloseTab,
     ReRunCommandInPane,
+    DropToShellInPane,
+    SpawnInPlaceTerminal,
+    DumpLayout,
+    LogLayoutToHd,
+    FillPluginCwd,
     Exit,
 }
 
@@ -380,6 +390,8 @@ pub enum PluginContext {
     PostMessageToPlugin,
     PluginSubscribedToEvents,
     PermissionRequestResult,
+    DumpLayout,
+    LogLayoutToHd,
 }
 
 /// Stack call representations corresponding to the different types of [`ClientInstruction`]s.
@@ -394,10 +406,12 @@ pub enum ClientContext {
     Connected,
     ActiveClients,
     Log,
+    LogError,
     OwnClientId,
     StartedParsingStdinQuery,
     DoneParsingStdinQuery,
     SwitchSession,
+    SetSynchronisedOutput,
 }
 
 /// Stack call representations corresponding to the different types of [`ServerInstruction`]s.
@@ -434,6 +448,9 @@ pub enum BackgroundJobContext {
     StopPluginLoadingAnimation,
     ReadAllSessionInfosOnMachine,
     ReportSessionInfo,
+    ReportLayoutInfo,
+    RunCommand,
+    WebRequest,
     Exit,
 }
 
