@@ -8,9 +8,9 @@ use super::layout::{
 use crate::cli::CliAction;
 use crate::data::InputMode;
 use crate::data::{Direction, Resize};
+use crate::home::{find_default_config_dir, get_layout_dir};
 use crate::input::config::{Config, ConfigError, KdlError};
 use crate::input::options::OnForceClose;
-use crate::setup::{find_default_config_dir, get_layout_dir};
 use miette::{NamedSource, Report};
 use serde::{Deserialize, Serialize};
 
@@ -124,6 +124,8 @@ pub enum Action {
     ClearScreen,
     /// Dumps the screen to a file
     DumpScreen(String, bool),
+    /// Dumps
+    DumpLayout,
     /// Scroll up in focus pane.
     EditScrollback,
     ScrollUp,
@@ -161,11 +163,14 @@ pub enum Action {
         Option<PathBuf>,
         Option<Direction>,
         bool,
-    ), // usize is an optional line number, Option<PathBuf> is an optional cwd, bool is floating true/false
+        bool,
+    ), // usize is an optional line number, Option<PathBuf> is an optional cwd, bool is floating true/false, second bool is in_place
     /// Open a new floating pane
     NewFloatingPane(Option<RunCommandAction>, Option<String>), // String is an optional pane name
     /// Open a new tiled (embedded, non-floating) pane
     NewTiledPane(Option<Direction>, Option<RunCommandAction>, Option<String>), // String is an
+    /// Open a new pane in place of the focused one, suppressing it instead
+    NewInPlacePane(Option<RunCommandAction>, Option<String>), // String is an
     // optional pane
     // name
     /// Embed focused pane in tab if floating or float focused pane if embedded
@@ -204,7 +209,8 @@ pub enum Action {
     LeftClick(Position),
     RightClick(Position),
     MiddleClick(Position),
-    LaunchOrFocusPlugin(RunPlugin, bool, bool), // bools => should float, move_to_focused_tab
+    LaunchOrFocusPlugin(RunPlugin, bool, bool, bool), // bools => should float,
+    // move_to_focused_tab, should_open_in_place
     LeftMouseRelease(Position),
     RightMouseRelease(Position),
     MiddleMouseRelease(Position),
@@ -232,6 +238,7 @@ pub enum Action {
     /// Open a new tiled (embedded, non-floating) plugin pane
     NewTiledPluginPane(RunPlugin, Option<String>), // String is an optional name
     NewFloatingPluginPane(RunPlugin, Option<String>), // String is an optional name
+    NewInPlacePluginPane(RunPlugin, Option<String>),  // String is an optional name
     StartOrReloadPlugin(RunPlugin),
     CloseTerminalPane(u32),
     ClosePluginPane(u32),
@@ -243,6 +250,7 @@ pub enum Action {
     BreakPane,
     BreakPaneRight,
     BreakPaneLeft,
+    RenameSession(String),
 }
 
 impl Action {
@@ -275,6 +283,7 @@ impl Action {
                 path.as_os_str().to_string_lossy().into(),
                 full,
             )]),
+            CliAction::DumpLayout => Ok(vec![Action::DumpLayout]),
             CliAction::EditScrollback => Ok(vec![Action::EditScrollback]),
             CliAction::ScrollUp => Ok(vec![Action::ScrollUp]),
             CliAction::ScrollDown => Ok(vec![Action::ScrollDown]),
@@ -293,6 +302,7 @@ impl Action {
                 plugin,
                 cwd,
                 floating,
+                in_place,
                 name,
                 close_on_exit,
                 start_suspended,
@@ -313,6 +323,8 @@ impl Action {
                     };
                     if floating {
                         Ok(vec![Action::NewFloatingPluginPane(plugin, name)])
+                    } else if in_place {
+                        Ok(vec![Action::NewInPlacePluginPane(plugin, name)])
                     } else {
                         // it is intentional that a new tiled plugin pane cannot include a
                         // direction
@@ -342,6 +354,8 @@ impl Action {
                             Some(run_command_action),
                             name,
                         )])
+                    } else if in_place {
+                        Ok(vec![Action::NewInPlacePane(Some(run_command_action), name)])
                     } else {
                         Ok(vec![Action::NewTiledPane(
                             direction,
@@ -352,6 +366,8 @@ impl Action {
                 } else {
                     if floating {
                         Ok(vec![Action::NewFloatingPane(None, name)])
+                    } else if in_place {
+                        Ok(vec![Action::NewInPlacePane(None, name)])
                     } else {
                         Ok(vec![Action::NewTiledPane(direction, None, name)])
                     }
@@ -362,6 +378,7 @@ impl Action {
                 file,
                 line_number,
                 floating,
+                in_place,
                 cwd,
             } => {
                 let mut file = file;
@@ -380,6 +397,7 @@ impl Action {
                     cwd,
                     direction,
                     floating,
+                    in_place,
                 )])
             },
             CliAction::SwitchMode { input_mode } => {
@@ -501,6 +519,7 @@ impl Action {
             CliAction::LaunchOrFocusPlugin {
                 url,
                 floating,
+                in_place,
                 move_to_focused_tab,
                 configuration,
             } => {
@@ -516,8 +535,10 @@ impl Action {
                     run_plugin,
                     floating,
                     move_to_focused_tab,
+                    in_place,
                 )])
             },
+            CliAction::RenameSession { name } => Ok(vec![Action::RenameSession(name)]),
         }
     }
 }
