@@ -124,6 +124,29 @@ pub struct Options {
     #[clap(long, value_parser)]
     #[serde(default)]
     pub auto_layout: Option<bool>,
+
+    /// Whether sessions should be serialized to the HD so that they can be later resurrected,
+    /// default is true
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub session_serialization: Option<bool>,
+
+    /// Whether pane viewports are serialized along with the session, default is false
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub serialize_pane_viewport: Option<bool>,
+
+    /// Scrollback lines to serialize along with the pane viewport when serializing sessions, 0
+    /// defaults to the scrollback size. If this number is higher than the scrollback size, it will
+    /// also default to the scrollback size
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub scrollback_lines_to_serialize: Option<usize>,
+
+    /// Whether to use ANSI styled underlines
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub styled_underlines: Option<bool>,
 }
 
 #[derive(ArgEnum, Deserialize, Serialize, Debug, Clone, Copy, PartialEq)]
@@ -187,6 +210,14 @@ impl Options {
         let attach_to_session = other
             .attach_to_session
             .or_else(|| self.attach_to_session.clone());
+        let session_serialization = other.session_serialization.or(self.session_serialization);
+        let serialize_pane_viewport = other
+            .serialize_pane_viewport
+            .or(self.serialize_pane_viewport);
+        let scrollback_lines_to_serialize = other
+            .scrollback_lines_to_serialize
+            .or(self.scrollback_lines_to_serialize);
+        let styled_underlines = other.styled_underlines.or(self.styled_underlines);
 
         Options {
             simplified_ui,
@@ -209,6 +240,10 @@ impl Options {
             session_name,
             attach_to_session,
             auto_layout,
+            session_serialization,
+            serialize_pane_viewport,
+            scrollback_lines_to_serialize,
+            styled_underlines,
         }
     }
 
@@ -232,6 +267,10 @@ impl Options {
         let pane_frames = merge_bool(other.pane_frames, self.pane_frames);
         let auto_layout = merge_bool(other.auto_layout, self.auto_layout);
         let mirror_session = merge_bool(other.mirror_session, self.mirror_session);
+        let session_serialization =
+            merge_bool(other.session_serialization, self.session_serialization);
+        let serialize_pane_viewport =
+            merge_bool(other.serialize_pane_viewport, self.serialize_pane_viewport);
 
         let default_mode = other.default_mode.or(self.default_mode);
         let default_shell = other.default_shell.or_else(|| self.default_shell.clone());
@@ -252,6 +291,10 @@ impl Options {
         let attach_to_session = other
             .attach_to_session
             .or_else(|| self.attach_to_session.clone());
+        let scrollback_lines_to_serialize = other
+            .scrollback_lines_to_serialize
+            .or_else(|| self.scrollback_lines_to_serialize.clone());
+        let styled_underlines = other.styled_underlines.or(self.styled_underlines);
 
         Options {
             simplified_ui,
@@ -274,6 +317,10 @@ impl Options {
             session_name,
             attach_to_session,
             auto_layout,
+            session_serialization,
+            serialize_pane_viewport,
+            scrollback_lines_to_serialize,
+            styled_underlines,
         }
     }
 
@@ -333,6 +380,10 @@ impl From<CliOptions> for Options {
             session_name: opts.session_name,
             attach_to_session: opts.attach_to_session,
             auto_layout: opts.auto_layout,
+            session_serialization: opts.session_serialization,
+            serialize_pane_viewport: opts.serialize_pane_viewport,
+            scrollback_lines_to_serialize: opts.scrollback_lines_to_serialize,
+            styled_underlines: opts.styled_underlines,
             ..Default::default()
         }
     }
