@@ -4,6 +4,7 @@ use crate::data::{
     Resize, SessionInfo, TabInfo,
 };
 use crate::envs::EnvironmentVariables;
+use crate::home::{find_default_config_dir, get_layout_dir};
 use crate::input::config::{Config, ConfigError, KdlError};
 use crate::input::keybinds::Keybinds;
 use crate::input::layout::{Layout, PluginUserConfiguration, RunPlugin, RunPluginLocation};
@@ -11,7 +12,6 @@ use crate::input::options::{Clipboard, OnForceClose, Options};
 use crate::input::permission::{GrantedPermission, PermissionCache};
 use crate::input::plugins::{PluginConfig, PluginTag, PluginType, PluginsConfig};
 use crate::input::theme::{FrameConfig, Theme, Themes, UiConfig};
-use crate::setup::{find_default_config_dir, get_layout_dir};
 use kdl_layout_parser::KdlLayoutParser;
 use std::collections::{BTreeMap, HashMap, HashSet};
 use strum::IntoEnumIterator;
@@ -469,6 +469,7 @@ impl Action {
             },
             "MovePaneBackwards" => Ok(Action::MovePaneBackwards),
             "DumpScreen" => Ok(Action::DumpScreen(string, false)),
+            "DumpLayout" => Ok(Action::DumpLayout),
             "NewPane" => {
                 if string.is_empty() {
                     return Ok(Action::NewPane(None, None));
@@ -504,6 +505,7 @@ impl Action {
                     })?;
                 Ok(Action::Search(search_direction))
             },
+            "RenameSession" => Ok(Action::RenameSession(string)),
             _ => Err(ConfigError::new_kdl_error(
                 format!("Unsupported action: {}", action_name),
                 action_node.span().offset(),
@@ -756,6 +758,11 @@ impl TryFrom<(&KdlNode, &Options)> for Action {
                 action_arguments,
                 kdl_action
             ),
+            "DumpLayout" => parse_kdl_action_char_or_string_arguments!(
+                action_name,
+                action_arguments,
+                kdl_action
+            ),
             "NewPane" => parse_kdl_action_char_or_string_arguments!(
                 action_name,
                 action_arguments,
@@ -886,6 +893,9 @@ impl TryFrom<(&KdlNode, &Options)> for Action {
                 let floating = command_metadata
                     .and_then(|c_m| kdl_child_bool_value_for_entry(c_m, "floating"))
                     .unwrap_or(false);
+                let in_place = command_metadata
+                    .and_then(|c_m| kdl_child_bool_value_for_entry(c_m, "in_place"))
+                    .unwrap_or(false);
                 let run_command_action = RunCommandAction {
                     command: PathBuf::from(command),
                     args,
@@ -896,6 +906,8 @@ impl TryFrom<(&KdlNode, &Options)> for Action {
                 };
                 if floating {
                     Ok(Action::NewFloatingPane(Some(run_command_action), name))
+                } else if in_place {
+                    Ok(Action::NewInPlacePane(Some(run_command_action), name))
                 } else {
                     Ok(Action::NewTiledPane(
                         direction,
@@ -923,6 +935,9 @@ impl TryFrom<(&KdlNode, &Options)> for Action {
                 let move_to_focused_tab = command_metadata
                     .and_then(|c_m| kdl_child_bool_value_for_entry(c_m, "move_to_focused_tab"))
                     .unwrap_or(false);
+                let should_open_in_place = command_metadata
+                    .and_then(|c_m| kdl_child_bool_value_for_entry(c_m, "in_place"))
+                    .unwrap_or(false);
                 let current_dir = std::env::current_dir().unwrap_or_else(|_| PathBuf::from("."));
                 let location = RunPluginLocation::parse(&plugin_path, Some(current_dir))?;
                 let configuration = KdlLayoutParser::parse_plugin_user_configuration(&kdl_action)?;
@@ -935,6 +950,7 @@ impl TryFrom<(&KdlNode, &Options)> for Action {
                     run_plugin,
                     should_float,
                     move_to_focused_tab,
+                    should_open_in_place,
                 ))
             },
             "PreviousSwapLayout" => Ok(Action::PreviousSwapLayout),
@@ -942,6 +958,11 @@ impl TryFrom<(&KdlNode, &Options)> for Action {
             "BreakPane" => Ok(Action::BreakPane),
             "BreakPaneRight" => Ok(Action::BreakPaneRight),
             "BreakPaneLeft" => Ok(Action::BreakPaneLeft),
+            "RenameSession" => parse_kdl_action_char_or_string_arguments!(
+                action_name,
+                action_arguments,
+                kdl_action
+            ),
             _ => Err(ConfigError::new_kdl_error(
                 format!("Unsupported action: {}", action_name).into(),
                 kdl_action.span().offset(),
@@ -1414,6 +1435,18 @@ impl Options {
         let attach_to_session =
             kdl_property_first_arg_as_bool_or_error!(kdl_options, "attach_to_session")
                 .map(|(v, _)| v);
+        let session_serialization =
+            kdl_property_first_arg_as_bool_or_error!(kdl_options, "session_serialization")
+                .map(|(v, _)| v);
+        let serialize_pane_viewport =
+            kdl_property_first_arg_as_bool_or_error!(kdl_options, "serialize_pane_viewport")
+                .map(|(v, _)| v);
+        let scrollback_lines_to_serialize =
+            kdl_property_first_arg_as_i64_or_error!(kdl_options, "scrollback_lines_to_serialize")
+                .map(|(v, _)| v as usize);
+        let styled_underlines =
+            kdl_property_first_arg_as_bool_or_error!(kdl_options, "styled_underlines")
+                .map(|(v, _)| v);
         Ok(Options {
             simplified_ui,
             theme,
@@ -1435,6 +1468,10 @@ impl Options {
             session_name,
             attach_to_session,
             auto_layout,
+            session_serialization,
+            serialize_pane_viewport,
+            scrollback_lines_to_serialize,
+            styled_underlines,
         })
     }
 }
@@ -1446,7 +1483,7 @@ impl Layout {
         raw_swap_layouts: Option<(&str, &str)>, // raw_swap_layouts swap_layouts_file_name
         cwd: Option<PathBuf>,
     ) -> Result<Self, ConfigError> {
-        let mut kdl_layout_parser = KdlLayoutParser::new(raw_layout, cwd);
+        let mut kdl_layout_parser = KdlLayoutParser::new(raw_layout, cwd, file_name.clone());
         let layout = kdl_layout_parser.parse().map_err(|e| match e {
             ConfigError::KdlError(kdl_error) => {
                 ConfigError::KdlError(kdl_error.add_src(file_name, String::from(raw_layout)))
@@ -1782,7 +1819,7 @@ impl Themes {
         Ok(themes)
     }
 
-    pub fn from_string(raw_string: String) -> Result<Self, ConfigError> {
+    pub fn from_string(raw_string: &String) -> Result<Self, ConfigError> {
         let kdl_config: KdlDocument = raw_string.parse()?;
         let kdl_themes = kdl_config.get("themes").ok_or(ConfigError::new_kdl_error(
             "No theme node found in file".into(),
@@ -1796,8 +1833,13 @@ impl Themes {
     pub fn from_path(path_to_theme_file: PathBuf) -> Result<Self, ConfigError> {
         // String is the theme name
         let kdl_config = std::fs::read_to_string(&path_to_theme_file)
-            .map_err(|e| ConfigError::IoPath(e, path_to_theme_file.into()))?;
-        Themes::from_string(kdl_config)
+            .map_err(|e| ConfigError::IoPath(e, path_to_theme_file.clone()))?;
+        Themes::from_string(&kdl_config).map_err(|e| match e {
+            ConfigError::KdlError(kdl_error) => ConfigError::KdlError(
+                kdl_error.add_src(path_to_theme_file.display().to_string(), kdl_config),
+            ),
+            e => e,
+        })
     }
 
     pub fn from_dir(path_to_theme_dir: PathBuf) -> Result<Self, ConfigError> {
